@@ -1,7 +1,26 @@
-
-/*                       
+/*
        This file implements a wrapper to the BLZPACK package
+
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+      
+   SLEPc is free software: you can redistribute it and/or modify it under  the
+   terms of version 3 of the GNU Lesser General Public License as published by
+   the Free Software Foundation.
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   more details.
+
+   You  should have received a copy of the GNU Lesser General  Public  License
+   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+
 #include "src/eps/impls/blzpack/blzpackp.h"   /*I "slepceps.h" I*/
 
 const char* blzpack_error[33] = {
@@ -45,8 +64,7 @@ const char* blzpack_error[33] = {
 PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
 {
   PetscErrorCode ierr;
-  PetscInt       N, n;
-  int            listor, lrstor, ncuv, k1, k2, k3, k4;
+  PetscInt       N, n, listor, lrstor, ncuv, k1, k2, k3, k4;
   EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
   PetscTruth     flg;
   KSP            ksp;
@@ -60,22 +78,23 @@ PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
       SETERRQ(0,"Warning: BLZpack recommends that ncv be larger than min(nev+10,nev*2)");
   }
   else eps->ncv = PetscMin(eps->nev+10,eps->nev*2);
+  if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
   if (!eps->max_it) eps->max_it = PetscMax(1000,N);
 
   if (!eps->ishermitian)
     SETERRQ(PETSC_ERR_SUP,"Requested method is only available for Hermitian problems");
-  if (blz->slice) {
+  if (blz->slice || eps->isgeneralized) {
     ierr = PetscTypeCompare((PetscObject)eps->OP,STSINV,&flg);CHKERRQ(ierr);
     if (!flg)
-      SETERRQ(PETSC_ERR_SUP,"Shift-and-invert ST is needed for spectrum slicing");
+      SETERRQ(PETSC_ERR_SUP,"Shift-and-invert ST is needed for generalized problems or spectrum slicing");
     ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
     ierr = PetscTypeCompare((PetscObject)ksp,KSPPREONLY,&flg);CHKERRQ(ierr);
     if (!flg)
-      SETERRQ(PETSC_ERR_SUP,"Preonly KSP is needed for spectrum slicing");
+      SETERRQ(PETSC_ERR_SUP,"Preonly KSP is needed for generalized problems or spectrum slicing");
     ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
     ierr = PetscTypeCompare((PetscObject)pc,PCCHOLESKY,&flg);CHKERRQ(ierr);
     if (!flg)
-      SETERRQ(PETSC_ERR_SUP,"Cholesky PC is needed for spectrum slicing");
+      SETERRQ(PETSC_ERR_SUP,"Cholesky PC is needed for generalized problems or spectrum slicing");
   }
   if (eps->which!=EPS_SMALLEST_REAL)
     SETERRQ(1,"Wrong value of eps->which");
@@ -87,8 +106,8 @@ PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
 
   listor = 123+k1*12;
   ierr = PetscFree(blz->istor);CHKERRQ(ierr);
-  ierr = PetscMalloc((17+listor)*sizeof(int),&blz->istor);CHKERRQ(ierr);
-  blz->istor[14] = listor;
+  ierr = PetscMalloc((17+listor)*sizeof(PetscBLASInt),&blz->istor);CHKERRQ(ierr);
+  blz->istor[14] = PetscBLASIntCast(listor);
 
   if (blz->slice) lrstor = n*(k2*4+k1*2+k4)+k3;
   else lrstor = n*(k2*4+k1)+k3;
@@ -105,7 +124,12 @@ PetscErrorCode EPSSetUp_BLZPACK(EPS eps)
   ierr = PetscFree(blz->eig);CHKERRQ(ierr);
   ierr = PetscMalloc(2*eps->ncv*sizeof(PetscReal),&blz->eig);CHKERRQ(ierr);
 
-  ierr = EPSAllocateSolutionContiguous(eps);CHKERRQ(ierr);
+  if (eps->extraction) {
+     ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr);
+  }
+
+  ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
+  ierr = EPSDefaultGetWork(eps,1);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -116,7 +140,7 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
   PetscErrorCode ierr;
   EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
   PetscInt       n, nn;
-  int            i, nneig, lflag, nvopu;      
+  PetscBLASInt   i, nneig, lflag, nvopu;      
   Vec            x, y;                           
   PetscScalar    sigma,*pV;                      
   Mat            A;                              
@@ -126,31 +150,36 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
   PetscFunctionBegin;
 
   ierr = VecGetLocalSize(eps->vec_initial,&n); CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(eps->comm,n,PETSC_DECIDE,PETSC_NULL,&x);CHKERRQ(ierr);
-  ierr = VecCreateMPIWithArray(eps->comm,n,PETSC_DECIDE,PETSC_NULL,&y);CHKERRQ(ierr);
+  ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,n,PETSC_DECIDE,PETSC_NULL,&x);CHKERRQ(ierr);
+  ierr = VecCreateMPIWithArray(((PetscObject)eps)->comm,n,PETSC_DECIDE,PETSC_NULL,&y);CHKERRQ(ierr);
   ierr = VecGetArray(eps->V[0],&pV);CHKERRQ(ierr);
   
-  if (blz->slice) { ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr); }
-  else sigma = 0.0;              /* shift of origin */
+  if (eps->isgeneralized && !blz->slice) { 
+    ierr = STGetShift(eps->OP,&sigma);CHKERRQ(ierr); /* shift of origin */
+    blz->rstor[0]  = sigma;        /* lower limit of eigenvalue interval */
+    blz->rstor[1]  = sigma;        /* upper limit of eigenvalue interval */
+  } else {
+    sigma = 0.0;
+    blz->rstor[0]  = blz->initial; /* lower limit of eigenvalue interval */
+    blz->rstor[1]  = blz->final;   /* upper limit of eigenvalue interval */
+  }
   nneig = 0;                     /* no. of eigs less than sigma */
 
-  blz->istor[0]  = n;            /* number of rows of U, V, X*/
-  blz->istor[1]  = n;            /* leading dimension of U, V, X */
-  blz->istor[2]  = eps->nev;     /* number of required eigenpairs */
-  blz->istor[3]  = eps->ncv;     /* number of working eigenpairs */
+  blz->istor[0]  = PetscBLASIntCast(n); /* number of rows of U, V, X*/
+  blz->istor[1]  = PetscBLASIntCast(n); /* leading dimension of U, V, X */
+  blz->istor[2]  = PetscBLASIntCast(eps->nev); /* number of required eigenpairs */
+  blz->istor[3]  = PetscBLASIntCast(eps->ncv); /* number of working eigenpairs */
   blz->istor[4]  = blz->block_size;    /* number of vectors in a block */
   blz->istor[5]  = blz->nsteps;  /* maximun number of steps per run */
   blz->istor[6]  = 1;            /* number of starting vectors as input */
   blz->istor[7]  = 0;            /* number of eigenpairs given as input */
-  blz->istor[8]  = blz->slice;   /* problem type */
+  blz->istor[8]  = (blz->slice || eps->isgeneralized) ? 1 : 0;   /* problem type */
   blz->istor[9]  = blz->slice;   /* spectrum slicing */
-  blz->istor[10] = blz->slice;   /* solutions refinement (purify) */
+  blz->istor[10] = eps->isgeneralized ? 1 : 0;   /* solutions refinement (purify) */
   blz->istor[11] = 0;            /* level of printing */
   blz->istor[12] = 6;            /* file unit for output */
-  blz->istor[13] = MPI_Comm_c2f(eps->comm);    /* communicator */
+  blz->istor[13] = PetscBLASIntCast(MPI_Comm_c2f(((PetscObject)eps)->comm)); /* communicator */
 
-  blz->rstor[0]  = blz->initial; /* lower limit of eigenvalue interval */
-  blz->rstor[1]  = blz->final;   /* upper limit of eigenvalue interval */
   blz->rstor[2]  = eps->tol;     /* threshold for convergence */
 
   lflag = 0;           /* reverse communication interface flag */
@@ -166,12 +195,12 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
       for (i=0;i<nvopu;i++) {
         ierr = VecPlaceArray( x, blz->u+i*n );CHKERRQ(ierr);
         ierr = VecPlaceArray( y, blz->v+i*n );CHKERRQ(ierr);
-        if (blz->slice) { 
-          ierr = STApplyNoB( eps->OP, x, y ); CHKERRQ(ierr);
+        if (blz->slice || eps->isgeneralized) { 
+          ierr = STAssociatedKSPSolve( eps->OP, x, y );CHKERRQ(ierr);
 	} else {
           ierr = STApply( eps->OP, x, y ); CHKERRQ(ierr);
 	}
-        ierr = EPSOrthogonalize(eps,eps->nds,PETSC_NULL,eps->DS,y,PETSC_NULL,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
+        ierr = IPOrthogonalize(eps->ip,eps->nds,PETSC_NULL,eps->DS,y,PETSC_NULL,PETSC_NULL,PETSC_NULL,eps->work[0],PETSC_NULL);CHKERRQ(ierr);
         ierr = VecResetArray(x);CHKERRQ(ierr);
         ierr = VecResetArray(y);CHKERRQ(ierr);	
       }
@@ -190,24 +219,25 @@ PetscErrorCode EPSSolve_BLZPACK(EPS eps)
       for (i=0;i<nvopu;i++) {
         ierr = VecPlaceArray( x, blz->u+i*n );CHKERRQ(ierr);
         ierr = VecPlaceArray( y, blz->v+i*n );CHKERRQ(ierr);
-        ierr = STApplyB( eps->OP, x, y ); CHKERRQ(ierr);
+        ierr = IPApplyMatrix(eps->ip, x, y ); CHKERRQ(ierr);
         ierr = VecResetArray(x);CHKERRQ(ierr);
         ierr = VecResetArray(y);CHKERRQ(ierr);	
       }
       break;
     case 3:  
       /* update shift */
+      PetscInfo1(eps,"Factorization update (sigma=%g)\n",sigma);
       ierr = STSetShift(eps->OP,sigma);CHKERRQ(ierr);
       ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
       ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-      ierr = PCGetFactoredMatrix(pc,&A);CHKERRQ(ierr);
+      ierr = PCFactorGetMatrix(pc,&A);CHKERRQ(ierr);
       ierr = MatGetInertia(A,&nn,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
-      nneig = nn;
+      nneig = PetscBLASIntCast(nn);
       break;
     case 4:  
       /* copy the initial vector */
       ierr = VecPlaceArray(x,blz->v);CHKERRQ(ierr);
-      ierr = VecCopy(eps->vec_initial,x);CHKERRQ(ierr); 
+      ierr = EPSGetStartVector(eps,0,x,PETSC_NULL);CHKERRQ(ierr);
       ierr = VecResetArray(x);CHKERRQ(ierr);
       break;
     }
@@ -244,7 +274,7 @@ PetscErrorCode EPSBackTransform_BLZPACK(EPS eps)
   EPS_BLZPACK    *blz = (EPS_BLZPACK *)eps->data;
 
   PetscFunctionBegin;
-  if (!blz->slice) {
+  if (!blz->slice && !eps->isgeneralized) {
     ierr = EPSBackTransform_Default(eps);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
@@ -265,7 +295,7 @@ PetscErrorCode EPSDestroy_BLZPACK(EPS eps)
   ierr = PetscFree(blz->v);CHKERRQ(ierr);
   ierr = PetscFree(blz->eig);CHKERRQ(ierr);
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = EPSFreeSolutionContiguous(eps);CHKERRQ(ierr);
+  ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -319,7 +349,7 @@ PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
     ierr = EPSBlzpackSetInterval(eps,interval[0],interval[1]);CHKERRQ(ierr);
   }
 
-  if (blz->slice) {
+  if (blz->slice || eps->isgeneralized) {
     ierr = STSetType(eps->OP,STSINV);CHKERRQ(ierr);
     ierr = STGetKSP(eps->OP,&ksp);CHKERRQ(ierr);
     ierr = KSPSetType(ksp,KSPPREONLY);CHKERRQ(ierr);
@@ -334,7 +364,7 @@ PetscErrorCode EPSSetFromOptions_BLZPACK(EPS eps)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "EPSBlzpackSetBlockSize_BLZPACK"
-PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,int bs)
+PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,PetscInt bs)
 {
   EPS_BLZPACK *blz = (EPS_BLZPACK *) eps->data;;
 
@@ -342,7 +372,7 @@ PetscErrorCode EPSBlzpackSetBlockSize_BLZPACK(EPS eps,int bs)
   if (bs == PETSC_DEFAULT) blz->block_size = 3;
   else if (bs <= 0) { 
     SETERRQ(1, "Incorrect block size"); 
-  } else blz->block_size = bs;
+  } else blz->block_size = PetscBLASIntCast(bs);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -365,9 +395,9 @@ EXTERN_C_END
 
 .seealso: EPSBlzpackSetInterval()
 @*/
-PetscErrorCode EPSBlzpackSetBlockSize(EPS eps,int bs)
+PetscErrorCode EPSBlzpackSetBlockSize(EPS eps,PetscInt bs)
 {
-  PetscErrorCode ierr, (*f)(EPS,int);
+  PetscErrorCode ierr, (*f)(EPS,PetscInt);
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -383,12 +413,14 @@ EXTERN_C_BEGIN
 #define __FUNCT__ "EPSBlzpackSetInterval_BLZPACK"
 PetscErrorCode EPSBlzpackSetInterval_BLZPACK(EPS eps,PetscReal initial,PetscReal final)
 {
+  PetscErrorCode ierr;
   EPS_BLZPACK *blz = (EPS_BLZPACK *) eps->data;;
 
   PetscFunctionBegin;
   blz->initial    = initial;
   blz->final      = final;
   blz->slice      = 1;
+  ierr = STSetShift(eps->OP,initial);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -437,12 +469,13 @@ PetscErrorCode EPSBlzpackSetInterval(EPS eps,PetscReal initial,PetscReal final)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "EPSBlzpackSetNSteps_BLZPACK"
-PetscErrorCode EPSBlzpackSetNSteps_BLZPACK(EPS eps,int nsteps)
+PetscErrorCode EPSBlzpackSetNSteps_BLZPACK(EPS eps,PetscInt nsteps)
 {
   EPS_BLZPACK *blz = (EPS_BLZPACK *) eps->data;
 
   PetscFunctionBegin;
-  blz->nsteps = nsteps == PETSC_DEFAULT ? 0 : nsteps;
+  if (nsteps == PETSC_DEFAULT) blz->nsteps = 0;
+  else { blz->nsteps = PetscBLASIntCast(nsteps); }
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -465,9 +498,9 @@ EXTERN_C_END
    Level: advanced
 
 @*/
-PetscErrorCode EPSBlzpackSetNSteps(EPS eps,int nsteps)
+PetscErrorCode EPSBlzpackSetNSteps(EPS eps,PetscInt nsteps)
 {
-  PetscErrorCode ierr, (*f)(EPS,int);
+  PetscErrorCode ierr, (*f)(EPS,PetscInt);
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
