@@ -1,21 +1,33 @@
-
-/*                       
+/*
        This file implements a wrapper to the PRIMME library
-       Contributed by Eloy Romero
+
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+      
+   SLEPc is free software: you can redistribute it and/or modify it under  the
+   terms of version 3 of the GNU Lesser General Public License as published by
+   the Free Software Foundation.
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   more details.
+
+   You  should have received a copy of the GNU Lesser General  Public  License
+   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
-#include "src/eps/epsimpl.h"    /*I "slepceps.h" I*/
+#include "private/epsimpl.h"    /*I "slepceps.h" I*/
+#include "private/stimpl.h"
 
 EXTERN_C_BEGIN
 #include "primme.h"
 EXTERN_C_END
 
-#ifndef PETSC_USE_COMPLEX
-typedef double PRIMMEScalar;
-#else
-typedef Complex_Z PRIMMEScalar;
-#endif
-
 typedef struct {
   primme_params primme;           /* param struc */
   primme_preset_method method;    /* primme method */
@@ -24,11 +36,49 @@ typedef struct {
   Vec x,y;                        /* auxiliar vectors */ 
 } EPS_PRIMME;
 
-static void multMatvec_PRIMME(PRIMMEScalar *in, PRIMMEScalar *out, int *blockSize, primme_params *primme);
-static void applyPreconditioner_PRIMME(PRIMMEScalar *in, PRIMMEScalar *out, int *blockSize, struct primme_params *primme);
+const char *methodList[] = {
+  "dynamic",
+  "default_min_time",
+  "default_min_matvecs",
+  "arnoldi",
+  "gd",
+  "gd_plusk",
+  "gd_olsen_plusk",
+  "jd_olsen_plusk",
+  "rqi",
+  "jdqr",
+  "jdqmr",
+  "jdqmr_etol",
+  "subspace_iteration",
+  "lobpcg_orthobasis",
+  "lobpcg_orthobasis_window"
+};
+const char *precondList[] = {"none", "diagonal"};
+EPSPRIMMEMethod methodN[] = {
+  EPSPRIMME_DYNAMIC,
+  EPSPRIMME_DEFAULT_MIN_TIME,
+  EPSPRIMME_DEFAULT_MIN_MATVECS,
+  EPSPRIMME_ARNOLDI,
+  EPSPRIMME_GD,
+  EPSPRIMME_GD_PLUSK,
+  EPSPRIMME_GD_OLSEN_PLUSK,
+  EPSPRIMME_JD_OLSEN_PLUSK,
+  EPSPRIMME_RQI,
+  EPSPRIMME_JDQR,
+  EPSPRIMME_JDQMR,
+  EPSPRIMME_JDQMR_ETOL,
+  EPSPRIMME_SUBSPACE_ITERATION,
+  EPSPRIMME_LOBPCG_ORTHOBASIS,
+  EPSPRIMME_LOBPCG_ORTHOBASISW
+};
+EPSPRIMMEPrecond precondN[] = {EPSPRIMME_NONE, EPSPRIMME_DIAGONAL};
 
-static void par_GlobalSumDouble(void *sendBuf, void *recvBuf, int *count, primme_params *primme) {
-  MPI_Allreduce((double*)sendBuf, (double*)recvBuf, *count, MPI_DOUBLE, MPI_SUM, ((EPS)(primme->commInfo))->comm);
+static void multMatvec_PRIMME(void *in, void *out, int *blockSize, primme_params *primme);
+static void applyPreconditioner_PRIMME(void *in, void *out, int *blockSize, struct primme_params *primme);
+
+void par_GlobalSumDouble(void *sendBuf, void *recvBuf, int *count, primme_params *primme) {
+  PetscErrorCode ierr;
+  ierr = MPI_Allreduce((double*)sendBuf, (double*)recvBuf, *count, MPI_DOUBLE, MPI_SUM, ((PetscObject)(primme->commInfo))->comm);CHKERRABORT(((PetscObject)(primme->commInfo))->comm,ierr);
 }
 
 #undef __FUNCT__  
@@ -37,14 +87,14 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
 {
   PetscErrorCode ierr;
   PetscInt       N, n;
-  int            numProcs, procID;
+  PetscMPIInt    numProcs, procID;
   EPS_PRIMME     *ops = (EPS_PRIMME *)eps->data;
   primme_params  *primme = &(((EPS_PRIMME *)eps->data)->primme);
 
   PetscFunctionBegin;
 
-  MPI_Comm_size(eps->comm,&numProcs);
-  MPI_Comm_rank(eps->comm,&procID);
+  ierr = MPI_Comm_size(((PetscObject)eps)->comm,&numProcs);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(((PetscObject)eps)->comm,&procID);CHKERRQ(ierr);
   
   /* Check some constraints and set some default values */ 
   ierr = VecGetSize(eps->vec_initial,&N);CHKERRQ(ierr);
@@ -64,12 +114,11 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
   primme->numEvals = eps->nev; 
   primme->matrix = ops;
   primme->commInfo = eps;
-  primme->initSize = 1;
   primme->maxMatvecs = eps->max_it; 
   primme->eps = eps->tol; 
   primme->numProcs = numProcs; 
   primme->procID = procID;
-  primme->globalSumDouble = par_GlobalSumDouble;
+  primme->printLevel = 0;
 
   switch(eps->which) {
     case EPS_LARGEST_REAL:
@@ -85,20 +134,23 @@ PetscErrorCode EPSSetUp_PRIMME(EPS eps)
       break;   
   }
   
-  primme_set_method(ops->method, primme);
+  if (primme_set_method(ops->method, primme) < 0)
+    SETERRQ(PETSC_ERR_SUP,"PRIMME method not valid");
   
   /* If user sets ncv, maxBasisSize is modified. If not, ncv is set as maxBasisSize */
   if (eps->ncv) primme->maxBasisSize = eps->ncv;
   else eps->ncv = primme->maxBasisSize;
   if (eps->ncv < eps->nev+primme->maxBlockSize)  
     SETERRQ(PETSC_ERR_SUP,"PRIMME needs ncv >= nev+maxBlockSize");
+  if (eps->mpd) PetscInfo(eps,"Warning: parameter mpd ignored\n");
+
+  if (eps->extraction) {
+     ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr);
+  }
 
   /* Set workspace */
-  ierr = EPSAllocateSolutionContiguous(eps);CHKERRQ(ierr);
+  ierr = EPSAllocateSolution(eps);CHKERRQ(ierr);
 
-  /* Copy vec_initial to V[0] vector */
-  ierr = VecCopy(eps->vec_initial, eps->V[0]); CHKERRQ(ierr);
- 
   if (primme->correctionParams.precondition) {
     /* Calc reciprocal A diagonal */
     ierr = VecDuplicate(eps->vec_initial, &ops->w); CHKERRQ(ierr);
@@ -123,12 +175,20 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
   EPS_PRIMME     *ops = (EPS_PRIMME *)eps->data;
   PetscScalar    *a;
 #ifdef PETSC_USE_COMPLEX
-  int            i;
+  PetscInt       i;
   PetscReal      *evals;
 #endif
 
   PetscFunctionBegin;
 
+  /* Reset some parameters left from previous runs */
+  ops->primme.aNorm    = 0.0;
+  ops->primme.initSize = 1;
+  ops->primme.iseed[0] = -1;
+
+  /* Copy vec_initial to V[0] vector */
+  ierr = VecCopy(eps->vec_initial, eps->V[0]); CHKERRQ(ierr);
+ 
   /* Call PRIMME solver */
   ierr = VecGetArray(eps->V[0], &a); CHKERRQ(ierr);
 #ifndef PETSC_USE_COMPLEX
@@ -164,22 +224,20 @@ PetscErrorCode EPSSolve_PRIMME(EPS eps)
       break;
   }
 
-  eps->nconv = ops->primme.initSize>=0?ops->primme.initSize:0;
-  eps->reason = EPS_CONVERGED_TOL;
-  eps->its = ops->primme.stats.numMatvecs;
+  eps->nconv = ops->primme.initSize >= 0 ? ops->primme.initSize : 0;
+  eps->reason = eps->ncv >= eps->nev ? EPS_CONVERGED_TOL : EPS_DIVERGED_ITS;
+  eps->its = ops->primme.stats.numOuterIterations;
+  eps->OP->applys = ops->primme.stats.numMatvecs;
 
-#ifndef PETSC_USE_COMPLEX
-  ierr = PetscMemzero(eps->eigi, eps->nconv*sizeof(double)); CHKERRQ(ierr);
-#endif
   PetscFunctionReturn(0);
 }
 
 #undef __FUNCT__  
 #define __FUNCT__ "multMatvec_PRIMME"
-static void multMatvec_PRIMME(PRIMMEScalar *in, PRIMMEScalar *out, int *blockSize, primme_params *primme)
+static void multMatvec_PRIMME(void *in, void *out, int *blockSize, primme_params *primme)
 {
   PetscErrorCode ierr;
-  int            i, N = primme->n;
+  PetscInt       i, N = primme->n;
   EPS_PRIMME     *ops = (EPS_PRIMME *)primme->matrix; 
   Vec            x = ops->x;
   Vec            y = ops->y;
@@ -201,10 +259,10 @@ static void multMatvec_PRIMME(PRIMMEScalar *in, PRIMMEScalar *out, int *blockSiz
 
 #undef __FUNCT__  
 #define __FUNCT__ "applyPreconditioner_PRIMME"
-static void applyPreconditioner_PRIMME(PRIMMEScalar *in, PRIMMEScalar *out, int *blockSize, struct primme_params *primme)
+static void applyPreconditioner_PRIMME(void *in, void *out, int *blockSize, struct primme_params *primme)
 {
   PetscErrorCode ierr;
-  int            i, N = primme->n;
+  PetscInt       i, N = primme->n;
   EPS_PRIMME     *ops = (EPS_PRIMME *)primme->matrix; 
   Vec            x = ops->x;
   Vec            y = ops->y;
@@ -242,7 +300,7 @@ PetscErrorCode EPSDestroy_PRIMME(EPS eps)
   ierr = VecDestroy(ops->x);CHKERRQ(ierr);
   ierr = VecDestroy(ops->y);CHKERRQ(ierr);
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
-  ierr = EPSFreeSolutionContiguous(eps);CHKERRQ(ierr);
+  ierr = EPSFreeSolution(eps);CHKERRQ(ierr);
  
   PetscFunctionReturn(0);
 }
@@ -254,26 +312,9 @@ PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
   PetscErrorCode ierr;
   PetscTruth isascii;
   primme_params *primme = &((EPS_PRIMME *)eps->data)->primme;
-  const char *methodList[] = {
-    "default_min_time",
-    "default_min_matvecs",
-    "arnoldi",
-    "gd",
-    "gd_plusk",
-    "gd_olsen_plusk",
-    "jd_olsen_plusk",
-    "rqi",
-    "jdqr",
-    "jdqmr",
-    "jdqmr_etol",
-    "subspace_iteration",
-    "lobpcg_orthobasis",
-    "lobpcg_orthobasis_window"
-  };
-  const char *precondList[] = {"none", "diagonal"};
-  EPSPRIMMEMethod methodN;
-  EPSPRIMMEPrecond precondN;
-  
+  EPSPRIMMEMethod methodn;
+  EPSPRIMMEPrecond precondn;
+
   PetscFunctionBegin;
   ierr = PetscTypeCompare((PetscObject)viewer,PETSC_VIEWER_ASCII,&isascii);CHKERRQ(ierr);
   if (!isascii) {
@@ -281,10 +322,13 @@ PetscErrorCode EPSView_PRIMME(EPS eps,PetscViewer viewer)
   }
   
   ierr = PetscViewerASCIIPrintf(viewer,"PRIMME solver block size: %d\n",primme->maxBlockSize);CHKERRQ(ierr);
-  ierr = EPSPRIMMEGetMethod(eps, &methodN);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIPrintf(viewer,"PRIMME solver method: %s\n", methodList[methodN]);CHKERRQ(ierr);
-  ierr = EPSPRIMMEGetPrecond(eps, &precondN);CHKERRQ(ierr);
-  ierr = PetscViewerASCIIPrintf(viewer,"PRIMME solver preconditioning: %s\n", precondList[precondN]);CHKERRQ(ierr);
+  ierr = EPSPRIMMEGetMethod(eps, &methodn);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"PRIMME solver method: %s\n", methodList[methodn]);CHKERRQ(ierr);
+  ierr = EPSPRIMMEGetPrecond(eps, &precondn);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"PRIMME solver preconditioning: %s\n", precondList[precondn]);CHKERRQ(ierr);
+
+  /* Display PRIMME params */
+  primme_display_params(*primme);
 
   PetscFunctionReturn(0);
 }
@@ -297,40 +341,6 @@ PetscErrorCode EPSSetFromOptions_PRIMME(EPS eps)
   EPS_PRIMME    *ops = (EPS_PRIMME *)eps->data;
   PetscInt       op;
   PetscTruth     flg;
-  const char *methodList[] = {
-    "default_min_time",
-    "default_min_matvecs",
-    "arnoldi",
-    "gd",
-    "gd_plusk",
-    "gd_olsen_plusk",
-    "jd_olsen_plusk",
-    "rqi",
-    "jdqr",
-    "jdqmr",
-    "jdqmr_etol",
-    "subspace_iteration",
-    "lobpcg_orthobasis",
-    "lobpcg_orthobasis_window"
-  };
-  const char *precondList[] = {"none", "diagonal"};
-  EPSPRIMMEMethod methodN[] = {
-    EPSPRIMME_DEFAULT_MIN_TIME,
-    EPSPRIMME_DEFAULT_MIN_MATVECS,
-    EPSPRIMME_ARNOLDI,
-    EPSPRIMME_GD,
-    EPSPRIMME_GD_PLUSK,
-    EPSPRIMME_GD_OLSEN_PLUSK,
-    EPSPRIMME_JD_OlSEN_PLUSK,
-    EPSPRIMME_RQI,
-    EPSPRIMME_JDQR,
-    EPSPRIMME_JDQMR,
-    EPSPRIMME_JDQMR_ETOL,
-    EPSPRIMME_SUBSPACE_ITERATION,
-    EPSPRIMME_LOBPCG_ORTHOBASIS,
-    EPSPRIMME_LOBPCG_ORTHOBASIS_WINDOW
-  };
-  EPSPRIMMEPrecond precondN[] = {EPSPRIMME_NONE, EPSPRIMME_DIAGONAL};
 
   PetscFunctionBegin;
   
@@ -341,7 +351,7 @@ PetscErrorCode EPSSetFromOptions_PRIMME(EPS eps)
   if (flg) {ierr = EPSPRIMMESetBlockSize(eps,op);CHKERRQ(ierr);}
   op = 0;
   ierr = PetscOptionsEList("-eps_primme_method","set method for solving the eigenproblem",
-                           "EPSPRIMMESetMethod",methodList,14,methodList[0],&op,&flg); CHKERRQ(ierr);
+                           "EPSPRIMMESetMethod",methodList,15,methodList[1],&op,&flg); CHKERRQ(ierr);
   if (flg) {ierr = EPSPRIMMESetMethod(eps, methodN[op]);CHKERRQ(ierr);}
   ierr = PetscOptionsEList("-eps_primme_precond","set preconditioner type",
                            "EPSPRIMMESetPrecond",precondList,2,precondList[0],&op,&flg); CHKERRQ(ierr);
@@ -355,7 +365,7 @@ PetscErrorCode EPSSetFromOptions_PRIMME(EPS eps)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "EPSPRIMMESetBlockSize_PRIMME"
-PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,int bs)
+PetscErrorCode EPSPRIMMESetBlockSize_PRIMME(EPS eps,PetscInt bs)
 {
   EPS_PRIMME *ops = (EPS_PRIMME *) eps->data;
 
@@ -396,9 +406,9 @@ EXTERN_C_END
    Level: advanced
 .seealso: EPSPRIMMEGetBlockSize()
 @*/
-PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,int bs)
+PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,PetscInt bs)
 {
-  PetscErrorCode ierr, (*f)(EPS,int);
+  PetscErrorCode ierr, (*f)(EPS,PetscInt);
 
   PetscFunctionBegin;
   
@@ -414,7 +424,7 @@ PetscErrorCode EPSPRIMMESetBlockSize(EPS eps,int bs)
 EXTERN_C_BEGIN
 #undef __FUNCT__  
 #define __FUNCT__ "EPSPRIMMEGetBlockSize_PRIMME"
-PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,int *bs)
+PetscErrorCode EPSPRIMMEGetBlockSize_PRIMME(EPS eps,PetscInt *bs)
 {
   EPS_PRIMME *ops = (EPS_PRIMME *) eps->data;
 
@@ -442,9 +452,9 @@ EXTERN_C_END
    Level: advanced
 .seealso: EPSPRIMMESetBlockSize()
 @*/
-PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,int *bs)
+PetscErrorCode EPSPRIMMEGetBlockSize(EPS eps,PetscInt *bs)
 {
-  PetscErrorCode ierr, (*f)(EPS,int*);
+  PetscErrorCode ierr, (*f)(EPS,PetscInt*);
 
   PetscFunctionBegin;
   
@@ -468,7 +478,7 @@ PetscErrorCode EPSPRIMMESetMethod_PRIMME(EPS eps, EPSPRIMMEMethod method)
 
   if (method == PETSC_DEFAULT) ops->method = DEFAULT_MIN_TIME;
   else ops->method = (primme_preset_method)method;
-  
+
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -483,12 +493,12 @@ EXTERN_C_END
    Input Parameters:
 +  eps - the eigenproblem solver context
 -  method - method that will be used by PRIMME. It must be one of:
-    EPSPRIMME_DEFAULT_MIN_TIME(EPSPRIMME_JDQMR_ETOL),
+    EPSPRIMME_DYNAMIC, EPSPRIMME_DEFAULT_MIN_TIME(EPSPRIMME_JDQMR_ETOL),
     EPSPRIMME_DEFAULT_MIN_MATVECS(EPSPRIMME_GD_OLSEN_PLUSK), EPSPRIMME_ARNOLDI,
     EPSPRIMME_GD, EPSPRIMME_GD_PLUSK, EPSPRIMME_GD_OLSEN_PLUSK, 
-    EPSPRIMME_JD_OlSEN_PLUSK, EPSPRIMME_RQI, EPSPRIMME_JDQR, EPSPRIMME_JDQMR, 
+    EPSPRIMME_JD_OLSEN_PLUSK, EPSPRIMME_RQI, EPSPRIMME_JDQR, EPSPRIMME_JDQMR, 
     EPSPRIMME_JDQMR_ETOL, EPSPRIMME_SUBSPACE_ITERATION,
-    EPSPRIMME_LOBPCG_ORTHOBASIS, EPSPRIMME_LOBPCG_ORTHOBASIS_WINDOW
+    EPSPRIMME_LOBPCG_ORTHOBASIS, EPSPRIMME_LOBPCG_ORTHOBASISW
 
    Options Database Key:
 .  -eps_primme_set_method - Sets the method for the PRIMME library.
@@ -497,7 +507,8 @@ EXTERN_C_END
    If not set, the method defaults to EPSPRIMME_DEFAULT_MIN_TIME.
 
    Level: advanced
-.seealso: EPSPRIMMEGetMethod()  
+
+.seealso: EPSPRIMMEGetMethod(), EPSPRIMMEMethod
 @*/
 PetscErrorCode EPSPRIMMESetMethod(EPS eps, EPSPRIMMEMethod method)
 {
@@ -523,8 +534,9 @@ PetscErrorCode EPSPRIMMEGetMethod_PRIMME(EPS eps, EPSPRIMMEMethod *method)
 
   PetscFunctionBegin;
 
-  if (method) *method = (EPSPRIMMEMethod)ops->method;
-  
+  if (method)
+    *method = (EPSPRIMMEMethod)ops->method;
+
   PetscFunctionReturn(0);
 }
 EXTERN_C_END
@@ -541,15 +553,16 @@ EXTERN_C_END
     
    Output Parameters: 
 .  method - method that will be used by PRIMME. It must be one of:
-    EPSPRIMME_DEFAULT_MIN_TIME(EPSPRIMME_JDQMR_ETOL),
+    EPSPRIMME_DYNAMIC, EPSPRIMME_DEFAULT_MIN_TIME(EPSPRIMME_JDQMR_ETOL),
     EPSPRIMME_DEFAULT_MIN_MATVECS(EPSPRIMME_GD_OLSEN_PLUSK), EPSPRIMME_ARNOLDI,
     EPSPRIMME_GD, EPSPRIMME_GD_PLUSK, EPSPRIMME_GD_OLSEN_PLUSK, 
-    EPSPRIMME_JD_OlSEN_PLUSK, EPSPRIMME_RQI, EPSPRIMME_JDQR, EPSPRIMME_JDQMR, 
+    EPSPRIMME_JD_OLSEN_PLUSK, EPSPRIMME_RQI, EPSPRIMME_JDQR, EPSPRIMME_JDQMR, 
     EPSPRIMME_JDQMR_ETOL, EPSPRIMME_SUBSPACE_ITERATION,
-    EPSPRIMME_LOBPCG_ORTHOBASIS, EPSPRIMME_LOBPCG_ORTHOBASIS_WINDOW
+    EPSPRIMME_LOBPCG_ORTHOBASIS, EPSPRIMME_LOBPCG_ORTHOBASISW
 
     Level: advanced
-.seealso: EPSPRIMMESetMethod()
+
+.seealso: EPSPRIMMESetMethod(), EPSPRIMMEMethod
 @*/
 PetscErrorCode EPSPRIMMEGetMethod(EPS eps, EPSPRIMMEMethod *method)
 {
@@ -603,7 +616,8 @@ EXTERN_C_END
     The default is no preconditioning.
     
     Level: advanced
-.seealso: EPSPRIMMEGetPrecond()
+
+.seealso: EPSPRIMMEGetPrecond(), EPSPRIMMEPrecond
 @*/
 PetscErrorCode EPSPRIMMESetPrecond(EPS eps, EPSPRIMMEPrecond precond)
 {
@@ -653,7 +667,8 @@ EXTERN_C_END
    (diagonal preconditioning)
 
     Level: advanced
-.seealso: EPSPRIMMESetPrecond()
+
+.seealso: EPSPRIMMESetPrecond(), EPSPRIMMEPrecond
 @*/
 PetscErrorCode EPSPRIMMEGetPrecond(EPS eps, EPSPRIMMEPrecond *precond)
 {
@@ -694,8 +709,8 @@ PetscErrorCode EPSCreate_PRIMME(EPS eps)
 
   primme_initialize(&primme->primme);
   primme->primme.matrixMatvec = multMatvec_PRIMME;
-  primme->method = DEFAULT_MIN_TIME;
-   
+  primme->primme.globalSumDouble = par_GlobalSumDouble;
+  primme->method = (primme_preset_method)EPSPRIMME_DEFAULT_MIN_TIME;
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPRIMMESetBlockSize_C","EPSPRIMMESetBlockSize_PRIMME",EPSPRIMMESetBlockSize_PRIMME);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPRIMMESetMethod_C","EPSPRIMMESetMethod_PRIMME",EPSPRIMMESetMethod_PRIMME);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunctionDynamic((PetscObject)eps,"EPSPRIMMESetPrecond_C","EPSPRIMMESetPrecond_PRIMME",EPSPRIMMESetPrecond_PRIMME);CHKERRQ(ierr); 
