@@ -1,11 +1,31 @@
 /*
      The basic EPS routines, Create, View, etc. are here.
+
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+      
+   SLEPc is free software: you can redistribute it and/or modify it under  the
+   terms of version 3 of the GNU Lesser General Public License as published by
+   the Free Software Foundation.
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   more details.
+
+   You  should have received a copy of the GNU Lesser General  Public  License
+   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
-#include "src/eps/epsimpl.h"      /*I "slepceps.h" I*/
+
+#include "private/epsimpl.h"      /*I "slepceps.h" I*/
 
 PetscFList EPSList = 0;
 PetscCookie EPS_COOKIE = 0;
-PetscEvent EPS_SetUp = 0, EPS_Solve = 0, EPS_Orthogonalize = 0;
+PetscLogEvent EPS_SetUp = 0, EPS_Solve = 0, EPS_Dense = 0;
 
 #undef __FUNCT__  
 #define __FUNCT__ "EPSInitializePackage"
@@ -32,13 +52,13 @@ PetscErrorCode EPSInitializePackage(char *path) {
   if (initialized) PetscFunctionReturn(0);
   initialized = PETSC_TRUE;
   /* Register Classes */
-  ierr = PetscLogClassRegister(&EPS_COOKIE,"Eigenproblem Solver");CHKERRQ(ierr);
+  ierr = PetscCookieRegister("Eigenproblem Solver",&EPS_COOKIE);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = EPSRegisterAll(path);CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister(&EPS_SetUp,"EPSSetUp",EPS_COOKIE);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister(&EPS_Solve,"EPSSolve",EPS_COOKIE);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister(&EPS_Orthogonalize,"EPSOrthogonalize",EPS_COOKIE); CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("EPSSetUp",EPS_COOKIE,&EPS_SetUp);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("EPSSolve",EPS_COOKIE,&EPS_Solve);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("EPSDense",EPS_COOKIE,&EPS_Dense); CHKERRQ(ierr);
   /* Process info exclusions */
   ierr = PetscOptionsGetString(PETSC_NULL, "-log_info_exclude", logList, 256, &opt);CHKERRQ(ierr);
   if (opt) {
@@ -90,12 +110,13 @@ PetscErrorCode EPSInitializePackage(char *path) {
 PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  const char     *type, *which;
+  const EPSType  type;
+  const char     *extr, *which;
   PetscTruth     isascii;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(eps->comm);
+  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(((PetscObject)eps)->comm);
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_COOKIE,2);
   PetscCheckSameComm(eps,1,viewer,2);
 
@@ -112,6 +133,7 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       case EPS_GHEP:  type = "generalized " HERM " eigenvalue problem"; break;
       case EPS_NHEP:  type = "non-" HERM " eigenvalue problem"; break;
       case EPS_GNHEP: type = "generalized non-" HERM " eigenvalue problem"; break;
+      case EPS_PGNHEP: type = "generalized non-" HERM " eigenvalue problem with " HERM " positive definite B"; break;
       case 0:         type = "not yet set"; break;
       default: SETERRQ(1,"Wrong value of eps->problem_type");
     }
@@ -134,45 +156,43 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
       ierr = (*eps->ops->view)(eps,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
-    switch (eps->which) {
-      case EPS_LARGEST_MAGNITUDE:  which = "largest eigenvalues in magnitude"; break;
-      case EPS_SMALLEST_MAGNITUDE: which = "smallest eigenvalues in magnitude"; break;
-      case EPS_LARGEST_REAL:       which = "largest real parts"; break;
-      case EPS_SMALLEST_REAL:      which = "smallest real parts"; break;
-      case EPS_LARGEST_IMAGINARY:  which = "largest imaginary parts"; break;
-      case EPS_SMALLEST_IMAGINARY: which = "smallest imaginary parts"; break;
-      default: SETERRQ(1,"Wrong value of eps->which");
+    if (eps->extraction) {
+      switch (eps->extraction) {
+        case EPS_RITZ:             extr = "Rayleigh-Ritz"; break;
+        case EPS_HARMONIC:         extr = "harmonic Ritz"; break;
+        case EPS_REFINED:          extr = "refined Ritz"; break;
+        case EPS_REFINED_HARMONIC: extr = "refined harmonic Ritz"; break;
+        default: SETERRQ(1,"Wrong value of eps->extraction");
+      }
+      ierr = PetscViewerASCIIPrintf(viewer,"  extraction type: %s\n",extr);CHKERRQ(ierr);
+    }
+    ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: ");CHKERRQ(ierr);
+    if (eps->target_set) {
+#if !defined(PETSC_USE_COMPLEX)
+      ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g\n",eps->target);CHKERRQ(ierr);
+#else
+      ierr = PetscViewerASCIIPrintf(viewer,"closest to target: %g+%g i\n",PetscRealPart(eps->target),PetscImaginaryPart(eps->target));CHKERRQ(ierr);
+#endif
+    } else {
+      switch (eps->which) {
+        case EPS_LARGEST_MAGNITUDE:  which = "largest eigenvalues in magnitude"; break;
+        case EPS_SMALLEST_MAGNITUDE: which = "smallest eigenvalues in magnitude"; break;
+        case EPS_LARGEST_REAL:       which = "largest real parts"; break;
+        case EPS_SMALLEST_REAL:      which = "smallest real parts"; break;
+        case EPS_LARGEST_IMAGINARY:  which = "largest imaginary parts"; break;
+        case EPS_SMALLEST_IMAGINARY: which = "smallest imaginary parts"; break;
+        default: SETERRQ(1,"Wrong value of eps->which");
+      }
+      ierr = PetscViewerASCIIPrintf(viewer,"%s\n",which);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer,"  selected portion of the spectrum: %s\n",which);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of eigenvalues (nev): %d\n",eps->nev);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  number of column vectors (ncv): %d\n",eps->ncv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  maximum dimension of projected problem (mpd): %d\n",eps->mpd);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  maximum number of iterations: %d\n", eps->max_it);
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerance: %g\n",eps->tol);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization method: ");CHKERRQ(ierr);
-    switch (eps->orthog_type) {
-      case EPS_MGS_ORTH:
-        ierr = PetscViewerASCIIPrintf(viewer,"modified Gram-Schmidt\n");CHKERRQ(ierr);
-        break;
-      case EPS_CGS_ORTH:
-        ierr = PetscViewerASCIIPrintf(viewer,"classical Gram-Schmidt\n");CHKERRQ(ierr);
-        break;
-      default: SETERRQ(1,"Wrong value of eps->orth_type");
-    }
-    ierr = PetscViewerASCIIPrintf(viewer,"  orthogonalization refinement: ");CHKERRQ(ierr);
-    switch (eps->orthog_ref) {
-      case EPS_ORTH_REFINE_NEVER:
-        ierr = PetscViewerASCIIPrintf(viewer,"never\n");CHKERRQ(ierr);
-        break;
-      case EPS_ORTH_REFINE_IFNEEDED:
-        ierr = PetscViewerASCIIPrintf(viewer,"if needed (eta: %f)\n",eps->orthog_eta);CHKERRQ(ierr);
-        break;
-      case EPS_ORTH_REFINE_ALWAYS:
-        ierr = PetscViewerASCIIPrintf(viewer,"always\n");CHKERRQ(ierr);
-        break;
-      default: SETERRQ(1,"Wrong value of eps->orth_ref");
-    }
     ierr = PetscViewerASCIIPrintf(viewer,"  dimension of user-provided deflation space: %d\n",eps->nds);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
+    ierr = IPView(eps->ip,viewer); CHKERRQ(ierr);
     ierr = STView(eps->OP,viewer); CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   } else {
@@ -184,14 +204,6 @@ PetscErrorCode EPSView(EPS eps,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSPublish_Petsc"
-static PetscErrorCode EPSPublish_Petsc(PetscObject object)
-{
-  PetscFunctionBegin;
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "EPSCreate"
 /*@C
@@ -221,23 +233,24 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   PetscValidPointer(outeps,2);
   *outeps = 0;
 
-  PetscHeaderCreate(eps,_p_EPS,struct _EPSOps,EPS_COOKIE,-1,"EPS",comm,EPSDestroy,EPSView);
-  PetscLogObjectCreate(eps);
+  ierr = PetscHeaderCreate(eps,_p_EPS,struct _EPSOps,EPS_COOKIE,-1,"EPS",comm,EPSDestroy,EPSView);CHKERRQ(ierr);
   *outeps = eps;
 
-  eps->bops->publish   = EPSPublish_Petsc;
   ierr = PetscMemzero(eps->ops,sizeof(struct _EPSOps));CHKERRQ(ierr);
 
-  eps->type            = -1;
   eps->max_it          = 0;
   eps->nev             = 1;
   eps->ncv             = 0;
+  eps->mpd             = 0;
   eps->allocated_ncv   = 0;
   eps->nds             = 0;
-  eps->tol             = 0.0;
+  eps->tol             = 1e-7;
   eps->which           = EPS_LARGEST_MAGNITUDE;
+  eps->target          = 0.0;
+  eps->target_set      = PETSC_FALSE;
   eps->evecsavailable  = PETSC_FALSE;
   eps->problem_type    = (EPSProblemType)0;
+  eps->extraction      = (EPSExtraction)0;
   eps->solverclass     = (EPSClass)0;
 
   eps->vec_initial     = 0;
@@ -245,7 +258,6 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->V               = 0;
   eps->AV              = 0;
   eps->W               = 0;
-  eps->AW              = 0;
   eps->T               = 0;
   eps->DS              = 0;
   eps->ds_ortho        = PETSC_TRUE;
@@ -254,6 +266,7 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->errest          = 0;
   eps->errest_left     = 0;
   eps->OP              = 0;
+  eps->ip              = 0;
   eps->data            = 0;
   eps->nconv           = 0;
   eps->its             = 0;
@@ -263,17 +276,18 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
   eps->work            = 0;
   eps->isgeneralized   = PETSC_FALSE;
   eps->ishermitian     = PETSC_FALSE;
+  eps->ispositive      = PETSC_FALSE;
   eps->setupcalled     = 0;
   eps->reason          = EPS_CONVERGED_ITERATING;
 
   eps->numbermonitors  = 0;
 
-  eps->orthog_type     = EPS_CGS_ORTH;
-  eps->orthog_ref      = EPS_ORTH_REFINE_IFNEEDED;
-  eps->orthog_eta      = PETSC_DEFAULT;
-
   ierr = STCreate(comm,&eps->OP); CHKERRQ(ierr);
   PetscLogObjectParent(eps,eps->OP);
+  ierr = IPCreate(comm,&eps->ip); CHKERRQ(ierr);
+  ierr = IPSetOptionsPrefix(eps->ip,((PetscObject)eps)->prefix);
+  ierr = IPAppendOptionsPrefix(eps->ip,"eps_");
+  PetscLogObjectParent(eps,eps->ip);
   ierr = PetscPublishAll(eps);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -309,7 +323,7 @@ PetscErrorCode EPSCreate(MPI_Comm comm,EPS *outeps)
 
 .seealso: STSetType(), EPSType
 @*/
-PetscErrorCode EPSSetType(EPS eps,EPSType type)
+PetscErrorCode EPSSetType(EPS eps,const EPSType type)
 {
   PetscErrorCode ierr,(*r)(EPS);
   PetscTruth match;
@@ -327,7 +341,7 @@ PetscErrorCode EPSSetType(EPS eps,EPSType type)
     eps->data = 0;
   }
 
-  ierr = PetscFListFind(eps->comm,EPSList,type,(void (**)(void)) &r);CHKERRQ(ierr);
+  ierr = PetscFListFind(EPSList,((PetscObject)eps)->comm,type,(void (**)(void)) &r);CHKERRQ(ierr);
 
   if (!r) SETERRQ1(1,"Unknown EPS type given: %s",type);
 
@@ -356,11 +370,12 @@ PetscErrorCode EPSSetType(EPS eps,EPSType type)
 
 .seealso: EPSSetType()
 @*/
-PetscErrorCode EPSGetType(EPS eps,EPSType *type)
+PetscErrorCode EPSGetType(EPS eps,const EPSType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  *type = eps->type_name;
+  PetscValidPointer(type,2);
+  *type = ((PetscObject)eps)->type_name;
   PetscFunctionReturn(0);
 }
 
@@ -368,7 +383,7 @@ PetscErrorCode EPSGetType(EPS eps,EPSType *type)
    EPSRegisterDynamic - Adds a method to the eigenproblem solver package.
 
    Synopsis:
-   EPSRegisterDynamic(char *name_solver,char *path,char *name_create,int (*routine_create)(EPS))
+   EPSRegisterDynamic(char *name_solver,char *path,char *name_create,PetscErrorCode (*routine_create)(EPS))
 
    Not Collective
 
@@ -397,17 +412,22 @@ $     -eps_type my_solver
 
    Level: advanced
 
-   Environmental variables such as ${PETSC_ARCH}, ${SLEPC_DIR}, ${BOPT},
+   Environmental variables such as ${PETSC_ARCH}, ${PETSC_DIR}, ${PETSC_LIB_DIR},
    and others of the form ${any_environmental_variable} occuring in pathname will be 
    replaced with appropriate values.
 
-.seealso: EPSRegisterAll()
+.seealso: EPSRegisterDestroy(), EPSRegisterAll()
 
 M*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "EPSRegister"
-PetscErrorCode EPSRegister(const char *sname,const char *path,const char *name,int (*function)(EPS))
+/*@C
+  EPSRegister - See EPSRegisterDynamic()
+
+  Level: advanced
+@*/
+PetscErrorCode EPSRegister(const char *sname,const char *path,const char *name,PetscErrorCode (*function)(EPS))
 {
   PetscErrorCode ierr;
   char           fullname[256];
@@ -418,6 +438,28 @@ PetscErrorCode EPSRegister(const char *sname,const char *path,const char *name,i
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "EPSRegisterDestroy"
+/*@
+   EPSRegisterDestroy - Frees the list of EPS methods that were
+   registered by EPSRegisterDynamic().
+
+   Not Collective
+
+   Level: advanced
+
+.seealso: EPSRegisterDynamic(), EPSRegisterAll()
+@*/
+PetscErrorCode EPSRegisterDestroy(void)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscFListDestroy(&EPSList);CHKERRQ(ierr);
+  ierr = EPSRegisterAll(PETSC_NULL);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "EPSDestroy"
 /*@
@@ -438,12 +480,13 @@ PetscErrorCode EPSDestroy(EPS eps)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (--eps->refct > 0) PetscFunctionReturn(0);
+  if (--((PetscObject)eps)->refct > 0) PetscFunctionReturn(0);
 
   /* if memory was published with AMS then destroy it */
   ierr = PetscObjectDepublish(eps);CHKERRQ(ierr);
 
   ierr = STDestroy(eps->OP);CHKERRQ(ierr);
+  ierr = IPDestroy(eps->ip);CHKERRQ(ierr);
 
   if (eps->ops->destroy) {
     ierr = (*eps->ops->destroy)(eps); CHKERRQ(ierr);
@@ -462,13 +505,79 @@ PetscErrorCode EPSDestroy(EPS eps)
   }
 
   if (eps->nds > 0) {
-    ierr = VecDestroyVecs(eps->DS, eps->nds);
+    ierr = VecDestroyVecs(eps->DS, eps->nds);CHKERRQ(ierr);
   }
   
   ierr = PetscFree(eps->DSV);CHKERRQ(ierr);
 
-  PetscLogObjectDestroy(eps);
-  PetscHeaderDestroy(eps);
+  ierr = EPSMonitorCancel(eps);CHKERRQ(ierr);
+
+  ierr = PetscHeaderDestroy(eps);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSSetTarget"
+/*@
+   EPSSetTarget - Sets the value of the target.
+
+   Not collective
+
+   Input Parameters:
++  eps    - eigensolver context
+-  target - the value of the target
+
+   Notes:
+   The target is a scalar value used to determine the portion of the spectrum
+   of interest.
+
+   If the target is not specified, then eigenvalues are computed according to
+   the which parameter (see EPSSetWhichEigenpairs()).
+   
+   If the target is specified, then the sought-after eigenvalues are those
+   closest to the target.
+
+   Level: beginner
+
+.seealso: EPSGetTarget(), EPSSetWhichEigenpairs()
+@*/
+PetscErrorCode EPSSetTarget(EPS eps,PetscScalar target)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  eps->target = target;
+  eps->target_set = PETSC_TRUE;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSGetTarget"
+/*@
+   EPSGetTarget - Gets the value of the target.
+
+   Not collective
+
+   Input Parameter:
+.  eps - eigensolver context
+
+   Output Parameter:
+.  target - the value of the target
+
+   Level: beginner
+
+   Note:
+   If the target was not set by the user, then zero is returned.
+
+.seealso: EPSSetTarget()
+@*/
+PetscErrorCode EPSGetTarget(EPS eps,PetscScalar* target)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  if (target) {
+    if (eps->target_set) *target = eps->target;
+    else *target = 0.0;
+  }
   PetscFunctionReturn(0);
 }
 
@@ -500,9 +609,9 @@ PetscErrorCode EPSSetST(EPS eps,ST st)
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
   PetscValidHeaderSpecific(st,ST_COOKIE,2);
   PetscCheckSameComm(eps,1,st,2);
+  ierr = PetscObjectReference((PetscObject)st);CHKERRQ(ierr);
   ierr = STDestroy(eps->OP); CHKERRQ(ierr);
   eps->OP = st;
-  ierr = PetscObjectReference((PetscObject)eps->OP);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -528,10 +637,72 @@ PetscErrorCode EPSGetST(EPS eps, ST *st)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidPointer(st,2);
   *st = eps->OP;
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "EPSSetIP"
+/*@
+   EPSSetIP - Associates an inner product object to the
+   eigensolver. 
+
+   Collective on EPS
+
+   Input Parameters:
++  eps - eigensolver context obtained from EPSCreate()
+-  ip  - the inner product object
+
+   Note:
+   Use EPSGetIP() to retrieve the inner product context (for example,
+   to free it at the end of the computations).
+
+   Level: advanced
+
+.seealso: EPSGetIP()
+@*/
+PetscErrorCode EPSSetIP(EPS eps,IP ip)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidHeaderSpecific(ip,IP_COOKIE,2);
+  PetscCheckSameComm(eps,1,ip,2);
+  ierr = PetscObjectReference((PetscObject)ip);CHKERRQ(ierr);
+  ierr = IPDestroy(eps->ip); CHKERRQ(ierr);
+  eps->ip = ip;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSGetIP"
+/*@C
+   EPSGetIP - Obtain the inner product object associated
+   to the eigensolver object.
+
+   Not Collective
+
+   Input Parameters:
+.  eps - eigensolver context obtained from EPSCreate()
+
+   Output Parameter:
+.  ip - inner product context
+
+   Level: advanced
+
+.seealso: EPSSetIP()
+@*/
+PetscErrorCode EPSGetIP(EPS eps,IP *ip)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidPointer(ip,2);
+  *ip = eps->ip;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "EPSIsGeneralized"
 /*@
