@@ -3,8 +3,28 @@
      All routines are simply wrappers to LAPACK routines. Matrices passed in
      as arguments are assumed to be square matrices stored in column-major 
      format with a leading dimension equal to the number of rows.
+
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+      
+   SLEPc is free software: you can redistribute it and/or modify it under  the
+   terms of version 3 of the GNU Lesser General Public License as published by
+   the Free Software Foundation.
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   more details.
+
+   You  should have received a copy of the GNU Lesser General  Public  License
+   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
-#include "slepceps.h" /*I "slepceps.h" I*/
+
+#include "private/epsimpl.h" /*I "slepceps.h" I*/
 #include "slepcblaslapack.h"
 
 #undef __FUNCT__  
@@ -36,7 +56,7 @@
 
 .seealso: EPSDenseGNHEP(), EPSDenseHEP(), EPSDenseGHEP()
 @*/
-PetscErrorCode EPSDenseNHEP(int n,PetscScalar *A,PetscScalar *w,PetscScalar *wi,PetscScalar *V,PetscScalar *W)
+PetscErrorCode EPSDenseNHEP(PetscInt n_,PetscScalar *A,PetscScalar *w,PetscScalar *wi,PetscScalar *V,PetscScalar *W)
 {
 #if defined(SLEPC_MISSING_LAPACK_GEEVX)
   PetscFunctionBegin;
@@ -45,15 +65,18 @@ PetscErrorCode EPSDenseNHEP(int n,PetscScalar *A,PetscScalar *w,PetscScalar *wi,
   PetscErrorCode ierr;
   PetscReal      abnrm,*scale,dummy;
   PetscScalar    *work;
-  int            ilo,ihi,lwork = 4*n,info;
+  PetscBLASInt   ilo,ihi,n,lwork,info;
   const char     *jobvr,*jobvl;
 #if defined(PETSC_USE_COMPLEX)
   PetscReal      *rwork;
 #else
-  int            idummy;
+  PetscBLASInt   idummy;
 #endif 
 
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  lwork = PetscBLASIntCast(4*n_);
   if (V) jobvr = "V";
   else jobvr = "N";
   if (W) jobvl = "V";
@@ -62,15 +85,16 @@ PetscErrorCode EPSDenseNHEP(int n,PetscScalar *A,PetscScalar *w,PetscScalar *wi,
   ierr  = PetscMalloc(n*sizeof(PetscReal),&scale);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
   ierr  = PetscMalloc(2*n*sizeof(PetscReal),&rwork);CHKERRQ(ierr);
-  LAPACKgeevx_("B",jobvl,jobvr,"N",&n,A,&n,w,W,&n,V,&n,&ilo,&ihi,scale,&abnrm,&dummy,&dummy,work,&lwork,rwork,&info,1,1,1,1);
+  LAPACKgeevx_("B",jobvl,jobvr,"N",&n,A,&n,w,W,&n,V,&n,&ilo,&ihi,scale,&abnrm,&dummy,&dummy,work,&lwork,rwork,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack ZGEEVX %d",info);
   ierr = PetscFree(rwork);CHKERRQ(ierr);
 #else
-  LAPACKgeevx_("B",jobvl,jobvr,"N",&n,A,&n,w,wi,W,&n,V,&n,&ilo,&ihi,scale,&abnrm,&dummy,&dummy,work,&lwork,&idummy,&info,1,1,1,1);
+  LAPACKgeevx_("B",jobvl,jobvr,"N",&n,A,&n,w,wi,W,&n,V,&n,&ilo,&ihi,scale,&abnrm,&dummy,&dummy,work,&lwork,&idummy,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack DGEEVX %d",info);
 #endif 
   ierr = PetscFree(work);CHKERRQ(ierr);
   ierr = PetscFree(scale);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif 
 }
@@ -105,7 +129,7 @@ PetscErrorCode EPSDenseNHEP(int n,PetscScalar *A,PetscScalar *w,PetscScalar *wi,
 
 .seealso: EPSDenseNHEP(), EPSDenseHEP(), EPSDenseGHEP()
 @*/
-PetscErrorCode EPSDenseGNHEP(int n,PetscScalar *A,PetscScalar *B,PetscScalar *w,PetscScalar *wi,PetscScalar *V,PetscScalar *W)
+PetscErrorCode EPSDenseGNHEP(PetscInt n_,PetscScalar *A,PetscScalar *B,PetscScalar *w,PetscScalar *wi,PetscScalar *V,PetscScalar *W)
 {
 #if defined(SLEPC_MISSING_LAPACK_GGEVX)
   PetscFunctionBegin;
@@ -114,17 +138,25 @@ PetscErrorCode EPSDenseGNHEP(int n,PetscScalar *A,PetscScalar *B,PetscScalar *w,
   PetscErrorCode ierr;
   PetscReal      *rscale,*lscale,abnrm,bbnrm,dummy;
   PetscScalar    *alpha,*beta,*work;
-  int            i,ilo,ihi,idummy,info;
+  PetscInt       i;
+  PetscBLASInt   ilo,ihi,idummy,info,n;
   const char     *jobvr,*jobvl;
 #if defined(PETSC_USE_COMPLEX)
   PetscReal      *rwork;
-  int            lwork = 2*n;
+  PetscBLASInt   lwork;
 #else
   PetscReal      *alphai;
-  int            lwork = 6*n;
+  PetscBLASInt   lwork;
 #endif 
 
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+#if defined(PETSC_USE_COMPLEX)
+  lwork = PetscBLASIntCast(2*n_);
+#else
+  lwork = PetscBLASIntCast(6*n_);
+#endif
   if (V) jobvr = "V";
   else jobvr = "N";
   if (W) jobvl = "V";
@@ -136,7 +168,7 @@ PetscErrorCode EPSDenseGNHEP(int n,PetscScalar *A,PetscScalar *B,PetscScalar *w,
   ierr  = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
   ierr  = PetscMalloc(6*n*sizeof(PetscReal),&rwork);CHKERRQ(ierr);
-  LAPACKggevx_("B",jobvl,jobvr,"N",&n,A,&n,B,&n,alpha,beta,W,&n,V,&n,&ilo,&ihi,lscale,rscale,&abnrm,&bbnrm,&dummy,&dummy,work,&lwork,rwork,&idummy,&idummy,&info,1,1,1,1);
+  LAPACKggevx_("B",jobvl,jobvr,"N",&n,A,&n,B,&n,alpha,beta,W,&n,V,&n,&ilo,&ihi,lscale,rscale,&abnrm,&bbnrm,&dummy,&dummy,work,&lwork,rwork,&idummy,&idummy,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack ZGGEVX %d",info);
   for (i=0;i<n;i++) {
     w[i] = alpha[i]/beta[i];
@@ -144,7 +176,7 @@ PetscErrorCode EPSDenseGNHEP(int n,PetscScalar *A,PetscScalar *B,PetscScalar *w,
   ierr = PetscFree(rwork);CHKERRQ(ierr);
 #else
   ierr  = PetscMalloc(n*sizeof(PetscReal),&alphai);CHKERRQ(ierr);
-  LAPACKggevx_("B",jobvl,jobvr,"N",&n,A,&n,B,&n,alpha,alphai,beta,W,&n,V,&n,&ilo,&ihi,lscale,rscale,&abnrm,&bbnrm,&dummy,&dummy,work,&lwork,&idummy,&idummy,&info,1,1,1,1);
+  LAPACKggevx_("B",jobvl,jobvr,"N",&n,A,&n,B,&n,alpha,alphai,beta,W,&n,V,&n,&ilo,&ihi,lscale,rscale,&abnrm,&bbnrm,&dummy,&dummy,work,&lwork,&idummy,&idummy,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack DGGEVX %d",info);
   for (i=0;i<n;i++) {
     w[i] = alpha[i]/beta[i];
@@ -157,6 +189,7 @@ PetscErrorCode EPSDenseGNHEP(int n,PetscScalar *A,PetscScalar *B,PetscScalar *w,
   ierr = PetscFree(rscale);CHKERRQ(ierr);
   ierr = PetscFree(lscale);CHKERRQ(ierr);
   ierr = PetscFree(work);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
@@ -188,7 +221,7 @@ PetscErrorCode EPSDenseGNHEP(int n,PetscScalar *A,PetscScalar *B,PetscScalar *w,
 
 .seealso: EPSDenseNHEP(), EPSDenseGNHEP(), EPSDenseGHEP()
 @*/
-PetscErrorCode EPSDenseHEP(int n,PetscScalar *A,int lda,PetscReal *w,PetscScalar *V)
+PetscErrorCode EPSDenseHEP(PetscInt n_,PetscScalar *A,PetscInt lda_,PetscReal *w,PetscScalar *V)
 {
 #if defined(SLEPC_MISSING_LAPACK_SYEVR) || defined(SLEPC_MISSING_LAPACK_HEEVR)
   PetscFunctionBegin;
@@ -197,33 +230,44 @@ PetscErrorCode EPSDenseHEP(int n,PetscScalar *A,int lda,PetscReal *w,PetscScalar
   PetscErrorCode ierr;
   PetscReal      abstol = 0.0,vl,vu;
   PetscScalar    *work;
-  int            il,iu,m,*isuppz,*iwork,liwork = 10*n,info;
+  PetscBLASInt   il,iu,m,*isuppz,*iwork,n,lda,liwork,info;
   const char     *jobz;
 #if defined(PETSC_USE_COMPLEX)
   PetscReal      *rwork;
-  int            lwork = 18*n,lrwork = 24*n;
+  PetscBLASInt   lwork,lrwork;
 #else
-  int            lwork = 26*n;
+  PetscBLASInt   lwork;
 #endif 
 
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  lda = PetscBLASIntCast(lda_);
+  liwork = PetscBLASIntCast(10*n_);
+#if defined(PETSC_USE_COMPLEX)
+  lwork = PetscBLASIntCast(18*n_);
+  lrwork = PetscBLASIntCast(24*n_);
+#else
+  lwork = PetscBLASIntCast(26*n_);
+#endif
   if (V) jobz = "V";
   else jobz = "N";
-  ierr  = PetscMalloc(2*n*sizeof(int),&isuppz);CHKERRQ(ierr);
+  ierr  = PetscMalloc(2*n*sizeof(PetscBLASInt),&isuppz);CHKERRQ(ierr);
   ierr  = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
-  ierr  = PetscMalloc(liwork*sizeof(int),&iwork);CHKERRQ(ierr);
+  ierr  = PetscMalloc(liwork*sizeof(PetscBLASInt),&iwork);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
   ierr  = PetscMalloc(lrwork*sizeof(PetscReal),&rwork);CHKERRQ(ierr);
-  LAPACKsyevr_(jobz,"A","L",&n,A,&lda,&vl,&vu,&il,&iu,&abstol,&m,w,V,&n,isuppz,work,&lwork,rwork,&lrwork,iwork,&liwork,&info,1,1,1);
+  LAPACKsyevr_(jobz,"A","L",&n,A,&lda,&vl,&vu,&il,&iu,&abstol,&m,w,V,&n,isuppz,work,&lwork,rwork,&lrwork,iwork,&liwork,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack ZHEEVR %d",info);
   ierr = PetscFree(rwork);CHKERRQ(ierr);
 #else
-  LAPACKsyevr_(jobz,"A","L",&n,A,&lda,&vl,&vu,&il,&iu,&abstol,&m,w,V,&n,isuppz,work,&lwork,iwork,&liwork,&info,1,1,1);
+  LAPACKsyevr_(jobz,"A","L",&n,A,&lda,&vl,&vu,&il,&iu,&abstol,&m,w,V,&n,isuppz,work,&lwork,iwork,&liwork,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack DSYEVR %d",info);
 #endif 
   ierr = PetscFree(isuppz);CHKERRQ(ierr);
   ierr = PetscFree(work);CHKERRQ(ierr);
   ierr = PetscFree(iwork);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
@@ -255,7 +299,7 @@ PetscErrorCode EPSDenseHEP(int n,PetscScalar *A,int lda,PetscReal *w,PetscScalar
 
 .seealso: EPSDenseNHEP(), EPSDenseGNHEP(), EPSDenseHEP()
 @*/
-PetscErrorCode EPSDenseGHEP(int n,PetscScalar *A,PetscScalar *B,PetscReal *w,PetscScalar *V)
+PetscErrorCode EPSDenseGHEP(PetscInt n_,PetscScalar *A,PetscScalar *B,PetscReal *w,PetscScalar *V)
 {
 #if defined(SLEPC_MISSING_LAPACK_SYGVD) || defined(SLEPC_MISSING_LAPACK_HEGVD)
   PetscFunctionBegin;
@@ -263,29 +307,47 @@ PetscErrorCode EPSDenseGHEP(int n,PetscScalar *A,PetscScalar *B,PetscReal *w,Pet
 #else
   PetscErrorCode ierr;
   PetscScalar    *work;
-  int            itype = 1,*iwork,info,
-                 liwork = V ? 5*n+3 : 1;
+  PetscBLASInt   itype = 1,*iwork,info,n,
+                 liwork;
   const char     *jobz;
 #if defined(PETSC_USE_COMPLEX)
   PetscReal      *rwork;
-  int            lwork  = V ? n*n+2*n     : n+1,
-                 lrwork = V ? 2*n*n+5*n+1 : n;
+  PetscBLASInt   lwork,lrwork;
 #else
-  int            lwork  = V ? 2*n*n+6*n+1 : 2*n+1;
+  PetscBLASInt   lwork;
 #endif 
 
   PetscFunctionBegin;
-  if (V) jobz = "V";
-  else jobz = "N";   
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  if (V) {
+    jobz = "V";
+    liwork = PetscBLASIntCast(5*n_+3);
+#if defined(PETSC_USE_COMPLEX)
+    lwork  = PetscBLASIntCast(n_*n_+2*n_);
+    lrwork = PetscBLASIntCast(2*n_*n_+5*n_+1);
+#else
+    lwork  = PetscBLASIntCast(2*n_*n_+6*n_+1);
+#endif
+  } else {
+    jobz = "N";   
+    liwork = 1;
+#if defined(PETSC_USE_COMPLEX)
+    lwork  = PetscBLASIntCast(n_+1);
+    lrwork = PetscBLASIntCast(n_);
+#else
+    lwork  = PetscBLASIntCast(2*n_+1);
+#endif
+  }
   ierr  = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
-  ierr  = PetscMalloc(liwork*sizeof(int),&iwork);CHKERRQ(ierr);
+  ierr  = PetscMalloc(liwork*sizeof(PetscBLASInt),&iwork);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
   ierr  = PetscMalloc(lrwork*sizeof(PetscReal),&rwork);CHKERRQ(ierr);
-  LAPACKsygvd_(&itype,jobz,"U",&n,A,&n,B,&n,w,work,&lwork,rwork,&lrwork,iwork,&liwork,&info,1,1);
+  LAPACKsygvd_(&itype,jobz,"U",&n,A,&n,B,&n,w,work,&lwork,rwork,&lrwork,iwork,&liwork,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack ZHEGVD %d",info);
   ierr = PetscFree(rwork);CHKERRQ(ierr);
 #else
-  LAPACKsygvd_(&itype,jobz,"U",&n,A,&n,B,&n,w,work,&lwork,iwork,&liwork,&info,1,1);
+  LAPACKsygvd_(&itype,jobz,"U",&n,A,&n,B,&n,w,work,&lwork,iwork,&liwork,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack DSYGVD %d",info);
 #endif 
   if (V) {
@@ -293,6 +355,7 @@ PetscErrorCode EPSDenseGHEP(int n,PetscScalar *A,PetscScalar *B,PetscReal *w,Pet
   }
   ierr = PetscFree(work);CHKERRQ(ierr);
   ierr = PetscFree(iwork);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif 
 }
@@ -324,7 +387,7 @@ PetscErrorCode EPSDenseGHEP(int n,PetscScalar *A,PetscScalar *B,PetscReal *w,Pet
 
 .seealso: EPSDenseSchur(), EPSSortDenseSchur(), EPSDenseTridiagonal()
 @*/
-PetscErrorCode EPSDenseHessenberg(int n,int k,PetscScalar *A,int lda,PetscScalar *Q)
+PetscErrorCode EPSDenseHessenberg(PetscInt n_,PetscInt k,PetscScalar *A,PetscInt lda_,PetscScalar *Q)
 {
 #if defined(SLEPC_MISSING_LAPACK_GEHRD) || defined(SLEPC_MISSING_LAPACK_ORGHR) || defined(SLEPC_MISSING_LAPACK_UNGHR)
   PetscFunctionBegin;
@@ -332,13 +395,17 @@ PetscErrorCode EPSDenseHessenberg(int n,int k,PetscScalar *A,int lda,PetscScalar
 #else
   PetscScalar    *tau,*work;
   PetscErrorCode ierr;
-  int            i,j,ilo,lwork,info;
+  PetscInt       i,j;
+  PetscBLASInt   ilo,lwork,info,n,lda;
 
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  lda = PetscBLASIntCast(lda_);
   ierr = PetscMalloc(n*sizeof(PetscScalar),&tau);CHKERRQ(ierr);
   lwork = n;
   ierr = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
-  ilo = k+1;
+  ilo = PetscBLASIntCast(k+1);
   LAPACKgehrd_(&n,&ilo,&n,A,&lda,tau,work,&lwork,&info);
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xGEHRD %d",info);
   for (j=0;j<n-1;j++) {
@@ -351,6 +418,7 @@ PetscErrorCode EPSDenseHessenberg(int n,int k,PetscScalar *A,int lda,PetscScalar
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xORGHR %d",info);
   ierr = PetscFree(tau);CHKERRQ(ierr);
   ierr = PetscFree(work);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
@@ -396,25 +464,28 @@ PetscErrorCode EPSDenseHessenberg(int n,int k,PetscScalar *A,int lda,PetscScalar
 
 .seealso: EPSDenseHessenberg(), EPSSortDenseSchur(), EPSDenseTridiagonal()
 @*/
-PetscErrorCode EPSDenseSchur(int n,int k,PetscScalar *H,int ldh,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi)
+PetscErrorCode EPSDenseSchur(PetscInt n_,PetscInt k,PetscScalar *H,PetscInt ldh_,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi)
 {
 #if defined(SLEPC_MISSING_LAPACK_HSEQR)
   PetscFunctionBegin;
   SETERRQ(PETSC_ERR_SUP,"HSEQR - Lapack routine is unavailable.");
 #else
   PetscErrorCode ierr;
-  int ilo,lwork,info;
-  PetscScalar *work;
+  PetscBLASInt   ilo,lwork,info,n,ldh;
+  PetscScalar    *work;
 #if !defined(PETSC_USE_COMPLEX)
-  int j;
+  PetscInt       j;
 #endif
   
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  ldh = PetscBLASIntCast(ldh_);
   lwork = n;
   ierr = PetscMalloc(lwork*sizeof(PetscScalar),&work);CHKERRQ(ierr);
-  ilo = k+1;
+  ilo = PetscBLASIntCast(k+1);
 #if !defined(PETSC_USE_COMPLEX)
-  LAPACKhseqr_("S","V",&n,&ilo,&n,H,&ldh,wr,wi,Z,&n,work,&lwork,&info,1,1);
+  LAPACKhseqr_("S","V",&n,&ilo,&n,H,&ldh,wr,wi,Z,&n,work,&lwork,&info);
   for (j=0;j<k;j++) {
     if (j==n-1 || H[j*ldh+j+1] == 0.0) { 
       /* real eigenvalue */
@@ -431,11 +502,12 @@ PetscErrorCode EPSDenseSchur(int n,int k,PetscScalar *H,int ldh,PetscScalar *Z,P
     }
   }
 #else
-  LAPACKhseqr_("S","V",&n,&ilo,&n,H,&ldh,wr,Z,&n,work,&lwork,&info,1,1);
+  LAPACKhseqr_("S","V",&n,&ilo,&n,H,&ldh,wr,Z,&n,work,&lwork,&info);
 #endif
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xHSEQR %d",info);
 
   ierr = PetscFree(work);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
@@ -456,10 +528,8 @@ PetscErrorCode EPSDenseSchur(int n,int k,PetscScalar *H,int ldh,PetscScalar *Z,P
 
    Input/Output Parameters:
 +  T  - the upper (quasi-)triangular matrix
--  Z  - the orthogonal matrix of Schur vectors
-
-   Output Parameters:
-+  wr - pointer to the array to store the computed eigenvalues
+.  Z  - the orthogonal matrix of Schur vectors
+.  wr - pointer to the array to store the computed eigenvalues
 -  wi - imaginary part of the eigenvalues (only when using real numbers)
 
    Notes:
@@ -476,20 +546,24 @@ PetscErrorCode EPSDenseSchur(int n,int k,PetscScalar *H,int ldh,PetscScalar *Z,P
 
 .seealso: EPSDenseHessenberg(), EPSDenseSchur(), EPSDenseTridiagonal()
 @*/
-PetscErrorCode EPSSortDenseSchur(int n,int k,PetscScalar *T,int ldt,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi,EPSWhich which)
+PetscErrorCode EPSSortDenseSchur(PetscInt n_,PetscInt k,PetscScalar *T,PetscInt ldt_,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi,EPSWhich which)
 {
 #if defined(SLEPC_MISSING_LAPACK_TREXC)
   PetscFunctionBegin;
   SETERRQ(PETSC_ERR_SUP,"TREXC - Lapack routine is unavailable.");
 #else
-  int i,j,ifst,ilst,info,pos;
-#if !defined(PETSC_USE_COMPLEX)
-  PetscScalar *work;
   PetscErrorCode ierr;
+  PetscReal      value,v;
+  PetscInt       i,j;
+  PetscBLASInt   ifst,ilst,info,pos,n,ldt;
+#if !defined(PETSC_USE_COMPLEX)
+  PetscScalar    *work;
 #endif
-  PetscReal   value,v;
   
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  ldt = PetscBLASIntCast(ldt_);
 #if !defined(PETSC_USE_COMPLEX)
   ierr = PetscMalloc(n*sizeof(PetscScalar),&work);CHKERRQ(ierr);
 #endif
@@ -559,12 +633,12 @@ PetscErrorCode EPSSortDenseSchur(int n,int k,PetscScalar *T,int ldt,PetscScalar
 #endif
     }
     if (pos) {
-      ifst = pos + 1;
-      ilst = i + 1;
+      ifst = PetscBLASIntCast(pos + 1);
+      ilst = PetscBLASIntCast(i + 1);
 #if !defined(PETSC_USE_COMPLEX)
-      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,work,&info,1);
+      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,work,&info);
 #else
-      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,&info,1);
+      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,&info);
 #endif
       if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xTREXC %d",info);
       
@@ -596,6 +670,157 @@ PetscErrorCode EPSSortDenseSchur(int n,int k,PetscScalar *T,int ldt,PetscScalar
 #if !defined(PETSC_USE_COMPLEX)
   ierr = PetscFree(work);CHKERRQ(ierr);
 #endif
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+
+#endif 
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSSortDenseSchurTarget"
+/*@
+   EPSSortDenseSchurTarget - Reorders the Schur decomposition computed by
+   EPSDenseSchur().
+
+   Not Collective
+
+   Input Parameters:
++  n     - dimension of the matrix 
+.  k     - first active column
+.  ldt   - leading dimension of T
+.  target - the target value
+-  which - eigenvalue sort order
+
+   Input/Output Parameters:
++  T  - the upper (quasi-)triangular matrix
+.  Z  - the orthogonal matrix of Schur vectors
+.  wr - pointer to the array to store the computed eigenvalues
+-  wi - imaginary part of the eigenvalues (only when using real numbers)
+
+   Notes:
+   This function reorders the eigenvalues in wr,wi located in positions k
+   to n according to increasing distance to the target. The parameter which
+   is used to determine if distance is relative to magnitude, real axis,
+   or imaginary axis. The Schur decomposition Z*T*Z^T, is also reordered 
+   by means of rotations so that eigenvalues in the diagonal blocks of T 
+   follow the same order.
+
+   Both T and Z are overwritten.
+   
+   This routine uses LAPACK routines xTREXC.
+
+   Level: developer
+
+.seealso: EPSDenseHessenberg(), EPSDenseSchur(), EPSDenseTridiagonal()
+@*/
+PetscErrorCode EPSSortDenseSchurTarget(PetscInt n_,PetscInt k,PetscScalar *T,PetscInt ldt_,PetscScalar *Z,PetscScalar *wr,PetscScalar *wi,PetscScalar target,EPSWhich which)
+{
+#if defined(SLEPC_MISSING_LAPACK_TREXC)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_ERR_SUP,"TREXC - Lapack routine is unavailable.");
+#else
+  PetscErrorCode ierr;
+  PetscReal      value,v;
+  PetscInt       i,j;
+  PetscBLASInt   ifst,ilst,info,pos,n,ldt;
+#if !defined(PETSC_USE_COMPLEX)
+  PetscScalar    *work;
+#endif
+  
+  PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  ldt = PetscBLASIntCast(ldt_);
+#if !defined(PETSC_USE_COMPLEX)
+  ierr = PetscMalloc(n*sizeof(PetscScalar),&work);CHKERRQ(ierr);
+#endif
+  
+  for (i=k;i<n-1;i++) {
+    switch(which) {
+      case EPS_LARGEST_MAGNITUDE:
+        /* complex target only allowed if scalartype=complex */
+        value = SlepcAbsEigenvalue(wr[i]-target,wi[i]);
+        break;
+      case EPS_LARGEST_REAL:
+        value = PetscAbsReal(PetscRealPart(wr[i]-target));
+        break;
+      case EPS_LARGEST_IMAGINARY:
+#if !defined(PETSC_USE_COMPLEX)
+        /* complex target only allowed if scalartype=complex */
+        value = PetscAbsReal(wi[i]);
+#else
+        value = PetscAbsReal(PetscImaginaryPart(wr[i]-target));
+#endif
+        break;
+      default: SETERRQ(1,"Wrong value of which");
+    }
+    pos = 0;
+    for (j=i+1;j<n;j++) {
+      switch(which) {
+        case EPS_LARGEST_MAGNITUDE:
+          /* complex target only allowed if scalartype=complex */
+          v = SlepcAbsEigenvalue(wr[j]-target,wi[j]);
+          break;
+        case EPS_LARGEST_REAL:
+          v = PetscAbsReal(PetscRealPart(wr[j]-target));
+          break;
+        case EPS_LARGEST_IMAGINARY:
+#if !defined(PETSC_USE_COMPLEX)
+          /* complex target only allowed if scalartype=complex */
+          v = PetscAbsReal(wi[j]);
+#else
+          v = PetscAbsReal(PetscImaginaryPart(wr[j]-target));
+#endif
+          break;
+        default: SETERRQ(1,"Wrong value of which");
+      }
+      if (v < value) {
+        value = v;
+        pos = j;
+      }
+#if !defined(PETSC_USE_COMPLEX)
+      if (wi[j] != 0) j++;
+#endif
+    }
+    if (pos) {
+      ifst = PetscBLASIntCast(pos + 1);
+      ilst = PetscBLASIntCast(i + 1);
+#if !defined(PETSC_USE_COMPLEX)
+      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,work,&info);
+#else
+      LAPACKtrexc_("V",&n,T,&ldt,Z,&n,&ifst,&ilst,&info);
+#endif
+      if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack xTREXC %d",info);
+      
+      for (j=k;j<n;j++) {
+#if !defined(PETSC_USE_COMPLEX)
+        if (j==n-1 || T[j*ldt+j+1] == 0.0) { 
+          /* real eigenvalue */
+          wr[j] = T[j*ldt+j];
+          wi[j] = 0.0;
+        } else {
+          /* complex eigenvalue */
+          wr[j] = T[j*ldt+j];
+          wr[j+1] = T[j*ldt+j];
+          wi[j] = sqrt(PetscAbsReal(T[j*ldt+j+1])) *
+                  sqrt(PetscAbsReal(T[(j+1)*ldt+j]));
+          wi[j+1] = -wi[j];
+          j++;
+        }
+#else
+        wr[j] = T[j*(ldt+1)];
+#endif
+      }
+    }
+#if !defined(PETSC_USE_COMPLEX)
+    if (wi[i] != 0) i++;
+#endif
+  }
+  
+#if !defined(PETSC_USE_COMPLEX)
+  ierr = PetscFree(work);CHKERRQ(ierr);
+#endif
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 
 #endif 
@@ -626,42 +851,39 @@ PetscErrorCode EPSSortDenseSchur(int n,int k,PetscScalar *T,int ldt,PetscScalar
 
 .seealso: EPSDenseNHEP(), EPSDenseHEP(), EPSDenseGNHEP(), EPSDenseGHEP()
 @*/
-PetscErrorCode EPSDenseTridiagonal(int n,PetscScalar *A,int lda,PetscReal *w,PetscScalar *V)
+PetscErrorCode EPSDenseTridiagonal(PetscInt n_,PetscReal *D,PetscReal *E,PetscReal *w,PetscScalar *V)
 {
 #if defined(SLEPC_MISSING_LAPACK_STEVR)
   PetscFunctionBegin;
-  SETERRQ(PETSC_ERR_SUP,"DSTEVR - Lapack routine is unavailable.");
+  SETERRQ(PETSC_ERR_SUP,"STEVR - Lapack routine is unavailable.");
 #else
   PetscErrorCode ierr;
-  PetscReal      abstol = 0.0,vl,vu,*D,*E,*work;
-  int            i,il,iu,m,*isuppz,lwork = 20*n,*iwork,liwork = 10*n,info;
+  PetscReal      abstol = 0.0,vl,vu,*work;
+  PetscBLASInt   il,iu,m,*isuppz,n,lwork,*iwork,liwork,info;
   const char     *jobz;
 #if defined(PETSC_USE_COMPLEX)
-  int            j;
+  PetscInt       i,j;
   PetscReal      *VV;
 #endif
   
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
+  n = PetscBLASIntCast(n_);
+  lwork = PetscBLASIntCast(20*n_);
+  liwork = PetscBLASIntCast(10*n_);
   if (V) {
     jobz = "V";
 #if defined(PETSC_USE_COMPLEX)
     ierr = PetscMalloc(n*n*sizeof(PetscReal),&VV);CHKERRQ(ierr);
 #endif
   } else jobz = "N";
-  ierr = PetscMalloc(n*sizeof(PetscReal),&D);CHKERRQ(ierr);
-  ierr = PetscMalloc(n*sizeof(PetscReal),&E);CHKERRQ(ierr);
-  ierr = PetscMalloc(2*n*sizeof(int),&isuppz);CHKERRQ(ierr);
+  ierr = PetscMalloc(2*n*sizeof(PetscBLASInt),&isuppz);CHKERRQ(ierr);
   ierr = PetscMalloc(lwork*sizeof(PetscReal),&work);CHKERRQ(ierr);
-  ierr = PetscMalloc(liwork*sizeof(int),&iwork);CHKERRQ(ierr);
-  for (i=0;i<n-1;i++) {
-    D[i] = PetscRealPart(A[i*(lda+1)]);
-    E[i] = PetscRealPart(A[i*(lda+1)+1]);
-  }
-  D[n-1] = PetscRealPart(A[(n-1)*(lda+1)]);
+  ierr = PetscMalloc(liwork*sizeof(PetscBLASInt),&iwork);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-  LAPACKstevr_(jobz,"A",&n,D,E,&vl,&vu,&il,&iu,&abstol,&m,w,VV,&n,isuppz,work,&lwork,iwork,&liwork,&info,1,1);
+  LAPACKstevr_(jobz,"A",&n,D,E,&vl,&vu,&il,&iu,&abstol,&m,w,VV,&n,isuppz,work,&lwork,iwork,&liwork,&info);
 #else
-  LAPACKstevr_(jobz,"A",&n,D,E,&vl,&vu,&il,&iu,&abstol,&m,w,V,&n,isuppz,work,&lwork,iwork,&liwork,&info,1,1);
+  LAPACKstevr_(jobz,"A",&n,D,E,&vl,&vu,&il,&iu,&abstol,&m,w,V,&n,isuppz,work,&lwork,iwork,&liwork,&info);
 #endif
   if (info) SETERRQ1(PETSC_ERR_LIB,"Error in Lapack DSTEVR %d",info);
 #if defined(PETSC_USE_COMPLEX)
@@ -672,11 +894,10 @@ PetscErrorCode EPSDenseTridiagonal(int n,PetscScalar *A,int lda,PetscReal *w,Pet
     ierr = PetscFree(VV);CHKERRQ(ierr);
   }
 #endif
-  ierr = PetscFree(D);CHKERRQ(ierr);
-  ierr = PetscFree(E);CHKERRQ(ierr);
   ierr = PetscFree(isuppz);CHKERRQ(ierr);
   ierr = PetscFree(work);CHKERRQ(ierr);
   ierr = PetscFree(iwork);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(EPS_Dense,0,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
