@@ -1,8 +1,28 @@
 /*
       EPS routines related to options that can be set via the command-line 
       or procedurally.
+
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+      
+   SLEPc is free software: you can redistribute it and/or modify it under  the
+   terms of version 3 of the GNU Lesser General Public License as published by
+   the Free Software Foundation.
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   more details.
+
+   You  should have received a copy of the GNU Lesser General  Public  License
+   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
-#include "src/eps/epsimpl.h"   /*I "slepceps.h" I*/
+
+#include "private/epsimpl.h"   /*I "slepceps.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSetFromOptions"
@@ -28,16 +48,15 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
   PetscErrorCode ierr;
   char           type[256],monfilename[PETSC_MAX_PATH_LEN];
   PetscTruth     flg;
-  const char     *orth_list[3] = { "mgs" , "cgs", "ncgs" };
-  const char     *ref_list[3] = { "never" , "ifneeded", "always" };
-  PetscReal      eta;
-  PetscInt       i,orth_type,ref_type;
-  PetscViewer    monviewer;
+  PetscReal      r;
+  PetscScalar    s;
+  PetscInt       i,j,k;
+  PetscViewerASCIIMonitor monviewer;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  ierr = PetscOptionsBegin(eps->comm,eps->prefix,"Eigenproblem Solver (EPS) Options","EPS");CHKERRQ(ierr);
-    ierr = PetscOptionsList("-eps_type","Eigenproblem Solver method","EPSSetType",EPSList,(char*)(eps->type_name?eps->type_name:EPSKRYLOVSCHUR),type,256,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsBegin(((PetscObject)eps)->comm,((PetscObject)eps)->prefix,"Eigenproblem Solver (EPS) Options","EPS");CHKERRQ(ierr);
+    ierr = PetscOptionsList("-eps_type","Eigenproblem Solver method","EPSSetType",EPSList,(char*)(((PetscObject)eps)->type_name?((PetscObject)eps)->type_name:EPSKRYLOVSCHUR),type,256,&flg);CHKERRQ(ierr);
     if (flg) {
       ierr = EPSSetType(eps,type);CHKERRQ(ierr);
     }
@@ -48,62 +67,62 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
     if (flg) {ierr = EPSSetProblemType(eps,EPS_GHEP);CHKERRQ(ierr);}
     ierr = PetscOptionsTruthGroup("-eps_non_hermitian","non-hermitian eigenvalue problem","EPSSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) {ierr = EPSSetProblemType(eps,EPS_NHEP);CHKERRQ(ierr);}
-    ierr = PetscOptionsTruthGroupEnd("-eps_gen_non_hermitian","generalized non-hermitian eigenvalue problem","EPSSetProblemType",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsTruthGroup("-eps_gen_non_hermitian","generalized non-hermitian eigenvalue problem","EPSSetProblemType",&flg);CHKERRQ(ierr);
     if (flg) {ierr = EPSSetProblemType(eps,EPS_GNHEP);CHKERRQ(ierr);}
+    ierr = PetscOptionsTruthGroupEnd("-eps_pos_gen_non_hermitian","generalized non-hermitian eigenvalue problem with positive semi-definite B","EPSSetProblemType",&flg);CHKERRQ(ierr);
+    if (flg) {ierr = EPSSetProblemType(eps,EPS_PGNHEP);CHKERRQ(ierr);}
 
     /*
       Set the type if it was never set.
     */
-    if (!eps->type_name) {
+    if (!((PetscObject)eps)->type_name) {
       ierr = EPSSetType(eps,EPSKRYLOVSCHUR);CHKERRQ(ierr);
     }
 
+    ierr = PetscOptionsTruthGroupBegin("-eps_ritz","Rayleigh-Ritz extraction","EPSSetExtraction",&flg);CHKERRQ(ierr);
+    if (flg) {ierr = EPSSetExtraction(eps,EPS_RITZ);CHKERRQ(ierr);}
+    ierr = PetscOptionsTruthGroup("-eps_harmonic","harmonic Ritz extraction","EPSSetExtraction",&flg);CHKERRQ(ierr);
+    if (flg) {ierr = EPSSetExtraction(eps,EPS_HARMONIC);CHKERRQ(ierr);}
+    ierr = PetscOptionsTruthGroup("-eps_refined","refined Ritz extraction","EPSSetExtraction",&flg);CHKERRQ(ierr);
+    if (flg) {ierr = EPSSetExtraction(eps,EPS_REFINED);CHKERRQ(ierr);}
+    ierr = PetscOptionsTruthGroupEnd("-eps_refined_harmonic","refined harmonic Ritz extraction","EPSSetExtraction",&flg);CHKERRQ(ierr);
+    if (flg) {ierr = EPSSetExtraction(eps,EPS_REFINED_HARMONIC);CHKERRQ(ierr);}
+
     ierr = PetscOptionsTruthGroupBegin("-eps_oneside","one-sided eigensolver","EPSSetClass",&flg);CHKERRQ(ierr);
     if (flg) {ierr = EPSSetClass(eps,EPS_ONE_SIDE);CHKERRQ(ierr);}
     ierr = PetscOptionsTruthGroupEnd("-eps_twoside","two-sided eigensolver","EPSSetClass",&flg);CHKERRQ(ierr);
     if (flg) {ierr = EPSSetClass(eps,EPS_TWO_SIDE);CHKERRQ(ierr);}
 
-    orth_type = eps->orthog_type;
-    ierr = PetscOptionsEList("-eps_orthog_type","Orthogonalization method","EPSSetOrthogonalization",orth_list,3,orth_list[eps->orthog_type],&orth_type,&flg);CHKERRQ(ierr);
-    ref_type = eps->orthog_ref;
-    ierr = PetscOptionsEList("-eps_orthog_refinement","Iterative refinement mode during orthogonalization","EPSSetOrthogonalizationRefinement",ref_list,3,ref_list[eps->orthog_ref],&ref_type,&flg);CHKERRQ(ierr);
-    eta = eps->orthog_eta;
-    ierr = PetscOptionsReal("-eps_orthog_eta","Parameter of iterative refinement during orthogonalization","EPSSetOrthogonalizationRefinement",eta,&eta,PETSC_NULL);CHKERRQ(ierr);
-    ierr = EPSSetOrthogonalization(eps,(EPSOrthogonalizationType)orth_type,(EPSOrthogonalizationRefinementType)ref_type,eta);CHKERRQ(ierr);
-
-    ierr = PetscOptionsInt("-eps_max_it","Maximum number of iterations","EPSSetTolerances",eps->max_it,&i,&flg);CHKERRQ(ierr);
-    if (flg) eps->max_it = i;
-    ierr = PetscOptionsReal("-eps_tol","Tolerance","KSPSetTolerances",eps->tol,&eps->tol,PETSC_NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsInt("-eps_nev","Number of eigenvalues to compute","EPSSetDimensions",eps->nev,&i,&flg);CHKERRQ(ierr);
-    if (flg) {
-      if(i<1) SETERRQ(1,"Illegal value for option -eps_nev. Must be > 0");
-      eps->nev = i;
-    }
-    ierr = PetscOptionsInt("-eps_ncv","Number of basis vectors","EPSSetDimensions",eps->ncv,&i,&flg);CHKERRQ(ierr);
-    if (flg) {
-      if (i<1) SETERRQ(1,"Illegal value for option -eps_ncv. Must be > 0");
-      eps->ncv = i;
-    }
+    r = i = PETSC_IGNORE;
+    ierr = PetscOptionsInt("-eps_max_it","Maximum number of iterations","EPSSetTolerances",eps->max_it,&i,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsReal("-eps_tol","Tolerance","EPSSetTolerances",eps->tol,&r,PETSC_NULL);CHKERRQ(ierr);
+    ierr = EPSSetTolerances(eps,r,i);CHKERRQ(ierr);
 
+    i = j = k = PETSC_IGNORE;
+    ierr = PetscOptionsInt("-eps_nev","Number of eigenvalues to compute","EPSSetDimensions",eps->nev,&i,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-eps_ncv","Number of basis vectors","EPSSetDimensions",eps->ncv,&j,PETSC_NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-eps_mpd","Maximum dimension of projected problem","EPSSetDimensions",eps->mpd,&k,PETSC_NULL);CHKERRQ(ierr);
+    ierr = EPSSetDimensions(eps,i,j,k);CHKERRQ(ierr);
+    
     /* -----------------------------------------------------------------------*/
     /*
       Cancels all monitors hardwired into code before call to EPSSetFromOptions()
     */
-    ierr = PetscOptionsName("-eps_cancelmonitors","Remove any hardwired monitor routines","EPSClearMonitor",&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsName("-eps_monitor_cancel","Remove any hardwired monitor routines","EPSMonitorCancel",&flg);CHKERRQ(ierr);
     if (flg) {
-      ierr = EPSClearMonitor(eps); CHKERRQ(ierr);
+      ierr = EPSMonitorCancel(eps); CHKERRQ(ierr);
     }
     /*
       Prints approximate eigenvalues and error estimates at each iteration
     */
-    ierr = PetscOptionsString("-eps_monitor","Monitor approximate eigenvalues and error estimates","EPSSetMonitor","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
+    ierr = PetscOptionsString("-eps_monitor","Monitor approximate eigenvalues and error estimates","EPSMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr); 
     if (flg) {
-      ierr = PetscViewerASCIIOpen(eps->comm,monfilename,&monviewer);CHKERRQ(ierr);
-      ierr = EPSSetMonitor(eps,EPSDefaultMonitor,monviewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIMonitorCreate(((PetscObject)eps)->comm,monfilename,((PetscObject)eps)->tablevel,&monviewer);CHKERRQ(ierr);
+      ierr = EPSMonitorSet(eps,EPSMonitorDefault,monviewer,(PetscErrorCode (*)(void*))PetscViewerASCIIMonitorDestroy);CHKERRQ(ierr);
     }
-    ierr = PetscOptionsName("-eps_xmonitor","Monitor error estimates graphically","EPSSetMonitor",&flg);CHKERRQ(ierr); 
+    ierr = PetscOptionsName("-eps_monitor_draw","Monitor error estimates graphically","EPSMonitorSet",&flg);CHKERRQ(ierr); 
     if (flg) {
-      ierr = EPSSetMonitor(eps,EPSLGMonitor,PETSC_NULL);CHKERRQ(ierr);
+      ierr = EPSMonitorSet(eps,EPSMonitorLG,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
     }
   /* -----------------------------------------------------------------------*/
     ierr = PetscOptionsTruthGroupBegin("-eps_largest_magnitude","compute largest eigenvalues in magnitude","EPSSetWhichEigenpairs",&flg);CHKERRQ(ierr);
@@ -119,6 +138,9 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
     ierr = PetscOptionsTruthGroupEnd("-eps_smallest_imaginary","compute smallest imaginary parts","EPSSetWhichEigenpairs",&flg);CHKERRQ(ierr);
     if (flg) {ierr = EPSSetWhichEigenpairs(eps,EPS_SMALLEST_IMAGINARY);CHKERRQ(ierr);}
 
+    ierr = PetscOptionsScalar("-eps_target","Value of the target","EPSSetTarget",eps->target,&s,&flg);CHKERRQ(ierr);
+    if (flg) {ierr = EPSSetTarget(eps,s);CHKERRQ(ierr);}
+
     ierr = PetscOptionsName("-eps_view","Print detailed information on solver used","EPSView",0);CHKERRQ(ierr);
     ierr = PetscOptionsName("-eps_view_binary","Save the matrices associated to the eigenproblem","EPSSetFromOptions",0);CHKERRQ(ierr);
     ierr = PetscOptionsName("-eps_plot_eigs","Make a plot of the computed eigenvalues","EPSSolve",0);CHKERRQ(ierr);
@@ -128,6 +150,7 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
     }
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
+  ierr = IPSetFromOptions(eps->ip); CHKERRQ(ierr);
   ierr = STSetFromOptions(eps->OP); CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -154,7 +177,7 @@ PetscErrorCode EPSSetFromOptions(EPS eps)
 
 .seealso: EPSSetTolerances()
 @*/
-PetscErrorCode EPSGetTolerances(EPS eps,PetscReal *tol,int *maxits)
+PetscErrorCode EPSGetTolerances(EPS eps,PetscReal *tol,PetscInt *maxits)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -181,18 +204,35 @@ PetscErrorCode EPSGetTolerances(EPS eps,PetscReal *tol,int *maxits)
 -  -eps_max_it <maxits> - Sets the maximum number of iterations allowed
 
    Notes:
-   Use PETSC_DEFAULT to retain the default value of any of the tolerances.
+   Use PETSC_IGNORE for an argument that need not be changed.
+
+   Use PETSC_DECIDE for maxits to assign a reasonably good value, which is 
+   dependent on the solution method.
 
    Level: intermediate
 
 .seealso: EPSGetTolerances()
 @*/
-PetscErrorCode EPSSetTolerances(EPS eps,PetscReal tol,int maxits)
+PetscErrorCode EPSSetTolerances(EPS eps,PetscReal tol,PetscInt maxits)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (tol != PETSC_DEFAULT)    eps->tol    = tol;
-  if (maxits != PETSC_DEFAULT) eps->max_it = maxits;
+  if (tol != PETSC_IGNORE) {
+    if (tol == PETSC_DEFAULT) {
+      eps->tol = 1e-7;
+    } else {
+      if (tol < 0.0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of tol. Must be > 0");
+      eps->tol = tol;
+    }
+  }
+  if (maxits != PETSC_IGNORE) {
+    if (maxits == PETSC_DEFAULT || maxits == PETSC_DECIDE) {
+      eps->max_it = 0;
+    } else {
+      if (maxits < 0) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of maxits. Must be > 0");
+      eps->max_it = maxits;
+    }
+  }
   PetscFunctionReturn(0);
 }
 
@@ -209,7 +249,8 @@ PetscErrorCode EPSSetTolerances(EPS eps,PetscReal tol,int maxits)
   
    Output Parameters:
 +  nev - number of eigenvalues to compute
--  ncv - the maximum dimension of the subspace to be used by the solver
+.  ncv - the maximum dimension of the subspace to be used by the solver
+-  mpd - the maximum dimension allowed for the projected problem
 
    Notes:
    The user can specify PETSC_NULL for any parameter that is not needed.
@@ -218,12 +259,13 @@ PetscErrorCode EPSSetTolerances(EPS eps,PetscReal tol,int maxits)
 
 .seealso: EPSSetDimensions()
 @*/
-PetscErrorCode EPSGetDimensions(EPS eps,int *nev,int *ncv)
+PetscErrorCode EPSGetDimensions(EPS eps,PetscInt *nev,PetscInt *ncv,PetscInt *mpd)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if( nev )   *nev = eps->nev;
-  if( ncv )   *ncv = eps->ncv;
+  if (nev) *nev = eps->nev;
+  if (ncv) *ncv = eps->ncv;
+  if (mpd) *mpd = eps->mpd;
   PetscFunctionReturn(0);
 }
 
@@ -238,40 +280,60 @@ PetscErrorCode EPSGetDimensions(EPS eps,int *nev,int *ncv)
    Input Parameters:
 +  eps - the eigensolver context
 .  nev - number of eigenvalues to compute
--  ncv - the maximum dimension of the subspace to be used by the solver
+.  ncv - the maximum dimension of the subspace to be used by the solver
+-  mpd - the maximum dimension allowed for the projected problem
 
    Options Database Keys:
 +  -eps_nev <nev> - Sets the number of eigenvalues
--  -eps_ncv <ncv> - Sets the dimension of the subspace
+.  -eps_ncv <ncv> - Sets the dimension of the subspace
+-  -eps_mpd <mpd> - Sets the maximum projected dimension
 
    Notes:
-   Use PETSC_DEFAULT to retain the previous value of any parameter.
+   Use PETSC_IGNORE to retain the previous value of any parameter.
 
-   Use PETSC_DECIDE for ncv to assign a reasonably good value, which is 
+   Use PETSC_DECIDE for ncv and mpd to assign a reasonably good value, which is
    dependent on the solution method.
 
+   The parameters ncv and mpd are intimately related, so that the user is advised
+   to set one of them at most. Normal usage is the following:
++  - In cases where nev is small, the user sets ncv (a reasonable default is 2*nev).
+-  - In cases where nev is large, the user sets mpd.
+
+   The value of ncv should always be between nev and (nev+mpd), typically
+   ncv=nev+mpd. If nev is not too large, mpd=nev is a reasonable choice, otherwise
+   a smaller value should be used.
+
    Level: intermediate
 
 .seealso: EPSGetDimensions()
 @*/
-PetscErrorCode EPSSetDimensions(EPS eps,int nev,int ncv)
+PetscErrorCode EPSSetDimensions(EPS eps,PetscInt nev,PetscInt ncv,PetscInt mpd)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
 
-  if( nev != PETSC_DEFAULT ) {
-    if (nev<1) SETERRQ(1,"Illegal value of nev. Must be > 0");
+  if( nev != PETSC_IGNORE ) {
+    if (nev<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of nev. Must be > 0");
     eps->nev = nev;
     eps->setupcalled = 0;
   }
-  if( ncv != PETSC_DEFAULT ) {
-    if( ncv == PETSC_DECIDE ) eps->ncv = 0;
-    else { 
-      if (ncv<1) SETERRQ(1,"Illegal value of ncv. Must be > 0");
+  if( ncv != PETSC_IGNORE ) {
+    if (ncv == PETSC_DECIDE || ncv == PETSC_DEFAULT) {
+      eps->ncv = 0;
+    } else {
+      if (ncv<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of ncv. Must be > 0");
       eps->ncv = ncv;
     }
     eps->setupcalled = 0;
   }
+  if( mpd != PETSC_IGNORE ) {
+    if (mpd == PETSC_DECIDE || mpd == PETSC_DEFAULT) {
+      eps->mpd = 0;
+    } else {
+      if (mpd<1) SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of mpd. Must be > 0");
+      eps->mpd = mpd;
+    }
+  }
   PetscFunctionReturn(0);
 }
 
@@ -316,13 +378,27 @@ PetscErrorCode EPSSetDimensions(EPS eps,int nev,int ncv)
     
     Level: intermediate
 
-.seealso: EPSGetWhichEigenpairs(), EPSSortEigenvalues()
+.seealso: EPSGetWhichEigenpairs(), EPSSortEigenvalues(), EPSWhich
 @*/
 PetscErrorCode EPSSetWhichEigenpairs(EPS eps,EPSWhich which)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  eps->which = which;
+  switch (which) {
+    case EPS_LARGEST_MAGNITUDE:
+    case EPS_SMALLEST_MAGNITUDE:
+    case EPS_LARGEST_REAL:
+    case EPS_SMALLEST_REAL:
+    case EPS_LARGEST_IMAGINARY:
+    case EPS_SMALLEST_IMAGINARY:
+      if (eps->which != which) {
+        eps->setupcalled = 0;
+        eps->which = which;
+      }
+      break;
+    default:
+      SETERRQ(PETSC_ERR_ARG_OUTOFRANGE,"Invalid 'which' value"); 
+  }  
   PetscFunctionReturn(0);
 }
 
@@ -345,12 +421,13 @@ PetscErrorCode EPSSetWhichEigenpairs(EPS eps,EPSWhich which)
 
     Level: intermediate
 
-.seealso: EPSSetWhichEigenpairs()
+.seealso: EPSSetWhichEigenpairs(), EPSWhich
 @*/
 PetscErrorCode EPSGetWhichEigenpairs(EPS eps,EPSWhich *which) 
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidPointer(which,2);
   *which = eps->which;
   PetscFunctionReturn(0);
 }
@@ -370,9 +447,11 @@ PetscErrorCode EPSGetWhichEigenpairs(EPS eps,EPSWhich *which)
 +  -eps_hermitian - Hermitian eigenvalue problem
 .  -eps_gen_hermitian - generalized Hermitian eigenvalue problem
 .  -eps_non_hermitian - non-Hermitian eigenvalue problem
--  -eps_gen_non_hermitian - generalized non-Hermitian eigenvalue problem 
+.  -eps_gen_non_hermitian - generalized non-Hermitian eigenvalue problem 
+-  -eps_pos_gen_non_hermitian - generalized non-Hermitian eigenvalue problem 
+   with positive semi-definite B
     
-   Note:  
+   Notes:  
    Allowed values for the problem type are: Hermitian (EPS_HEP), non-Hermitian
    (EPS_NHEP), generalized Hermitian (EPS_GHEP) and generalized non-Hermitian 
    (EPS_GNHEP).
@@ -386,39 +465,38 @@ PetscErrorCode EPSGetWhichEigenpairs(EPS eps,EPSWhich *which)
 
    Level: beginner
 
-.seealso: EPSSetOperators(), EPSSetType(), EPSProblemType
+.seealso: EPSSetOperators(), EPSSetType(), EPSGetProblemType(), EPSProblemType
 @*/
 PetscErrorCode EPSSetProblemType(EPS eps,EPSProblemType type)
 {
-  PetscErrorCode ierr;
-  Mat            A,B;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
 
-  ierr = STGetOperators(eps->OP,&A,&B);CHKERRQ(ierr);
-  if (!A) { SETERRQ(1,"Must call EPSSetOperators() first"); }
-
   switch (type) {
     case EPS_HEP:
       eps->isgeneralized = PETSC_FALSE;
       eps->ishermitian = PETSC_TRUE;
-      ierr = STSetBilinearForm(eps->OP,STINNER_HERMITIAN);CHKERRQ(ierr);
+      eps->ispositive = PETSC_FALSE;
       break;      
     case EPS_NHEP:
       eps->isgeneralized = PETSC_FALSE;
       eps->ishermitian = PETSC_FALSE;
-      ierr = STSetBilinearForm(eps->OP,STINNER_HERMITIAN);CHKERRQ(ierr);
+      eps->ispositive = PETSC_FALSE;
       break;
     case EPS_GHEP:
       eps->isgeneralized = PETSC_TRUE;
       eps->ishermitian = PETSC_TRUE;
-      ierr = STSetBilinearForm(eps->OP,STINNER_B_HERMITIAN);CHKERRQ(ierr);
+      eps->ispositive = PETSC_TRUE;
       break;
     case EPS_GNHEP:
       eps->isgeneralized = PETSC_TRUE;
       eps->ishermitian = PETSC_FALSE;
-      ierr = STSetBilinearForm(eps->OP,STINNER_HERMITIAN);CHKERRQ(ierr);
+      eps->ispositive = PETSC_FALSE;
+      break;
+    case EPS_PGNHEP:
+      eps->isgeneralized = PETSC_TRUE;
+      eps->ishermitian = PETSC_FALSE;
+      eps->ispositive = PETSC_TRUE;
       break;
 /*
     case EPS_CSEP: 
@@ -455,16 +533,82 @@ PetscErrorCode EPSSetProblemType(EPS eps,EPSProblemType type)
 
    Level: intermediate
 
-.seealso: EPSSetProblemType()
+.seealso: EPSSetProblemType(), EPSProblemType
 @*/
 PetscErrorCode EPSGetProblemType(EPS eps,EPSProblemType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidPointer(type,2);
   *type = eps->problem_type;
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "EPSSetExtraction"
+/*@
+   EPSSetExtraction - Specifies the type of extraction technique to be employed 
+   by the eigensolver.
+
+   Collective on EPS
+
+   Input Parameters:
++  eps  - the eigensolver context
+-  extr - a known type of extraction
+
+   Options Database Keys:
++  -eps_ritz - Rayleigh-Ritz extraction
+.  -eps_harmonic - hamonic Ritz extraction
+.  -eps_refined - refined Ritz extraction
+-  -eps_refined_harmonic - refined harmonic Ritz extraction
+    
+   Notes:  
+   Not all eigensolvers support all types of extraction. See the SLEPc
+   Users Manual for details.
+
+   By default, a standard Rayleigh-Ritz extraction is used. Other extractions
+   may be useful when computing interior eigenvalues.
+
+   Harmonic-type extractions are used in combination with a 'target'.
+
+   Level: beginner
+
+.seealso: EPSSetTarget(), EPSGetExtraction(), EPSExtraction
+@*/
+PetscErrorCode EPSSetExtraction(EPS eps,EPSExtraction extr)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  eps->extraction = extr;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNCT__  
+#define __FUNCT__ "EPSGetExtraction"
+/*@C
+   EPSGetExtraction - Gets the extraction type used by the EPS object.
+
+   Not Collective
+
+   Input Parameter:
+.  eps - the eigensolver context 
+
+   Output Parameter:
+.  extr - name of extraction type 
+
+   Level: intermediate
+
+.seealso: EPSSetExtraction(), EPSExtraction
+@*/
+PetscErrorCode EPSGetExtraction(EPS eps,EPSExtraction *extr)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidPointer(extr,2);
+  *extr = eps->extraction;
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSetClass"
 /*@
@@ -486,10 +630,10 @@ PetscErrorCode EPSGetProblemType(EPS eps,EPSProblemType *type)
    eigenvalues and (right) eigenvectors, whereas two-sided eigensolvers compute
    left eigenvectors as well.
 
-   Level: beginner
+   Level: intermediate
 
 .seealso: EPSGetLeftVector(), EPSComputeRelativeErrorLeft(), EPSSetLeftInitialVector(),
-   EPSClass
+   EPSGetClass(), EPSClass
 @*/
 PetscErrorCode EPSSetClass(EPS eps,EPSClass cl)
 {
@@ -522,109 +666,17 @@ PetscErrorCode EPSSetClass(EPS eps,EPSClass cl)
 
    Level: intermediate
 
-.seealso: EPSSetClass()
+.seealso: EPSSetClass(), EPSClass
 @*/
 PetscErrorCode EPSGetClass(EPS eps,EPSClass *cl)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidPointer(cl,2);
   *cl = eps->solverclass;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "EPSSetOrthogonalization"
-/*@
-   EPSSetOrthogonalization - Specifies the type of orthogonalization technique
-   to be used inside the eigensolver (classical or modified Gram-Schmidt with
-   or without refinement).
-
-   Collective on EPS
-
-   Input Parameters:
-+  eps        - the eigensolver context 
-.  type       - the type of orthogonalization technique
-.  refinement - type of refinement
--  eta        - parameter for selective refinement
-
-   Options Database Keys:
-+  -eps_orthog_type <type> -  Where <type> is cgs for Classical Gram-Schmidt
-                              orthogonalization (default)
-                              or mgs for Modified Gram-Schmidt orthogonalization
-.  -eps_orthog_refinement <type> -  Where <type> is one of never, ifneeded
-                              (default) or always 
--  -eps_orthog_eta <eta> -  For setting the value of eta
-    
-   Notes:  
-   The default settings work well for most problems. 
-
-   The parameter eta should be a real value between 0 and 1 (or PETSC_DEFAULT).
-   The value of eta is used only when the refinement type is "ifneeded". 
-
-   When using several processors, MGS is likely to result in bad scalability.
-
-   Level: advanced
-
-.seealso: EPSOrthogonalize(), EPSGetOrthogonalization()
-@*/
-PetscErrorCode EPSSetOrthogonalization(EPS eps,EPSOrthogonalizationType type, EPSOrthogonalizationRefinementType refinement, PetscReal eta)
-{
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  switch (type) {
-    case EPS_CGS_ORTH:
-    case EPS_MGS_ORTH:
-      eps->orthog_type = type;
-      break;
-    default:
-      SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown orthogonalization type");
-  }
-  switch (refinement) {
-    case EPS_ORTH_REFINE_NEVER:
-    case EPS_ORTH_REFINE_IFNEEDED:
-    case EPS_ORTH_REFINE_ALWAYS:
-      eps->orthog_ref = refinement;
-      break;
-    default:
-      SETERRQ(PETSC_ERR_ARG_WRONG,"Unknown refinement type");
-  }
-  if (eta != PETSC_DEFAULT && eta <= 0.0 && eta > 1.0) {
-    SETERRQ(PETSC_ERR_ARG_WRONG,"Invalid eta value");    
-  }
-  eps->orthog_eta = eta;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__  
-#define __FUNCT__ "EPSGetOrthogonalization"
-/*@C
-   EPSGetOrthogonalization - Gets the orthogonalization settings from the 
-   EPS object.
-
-   Not Collective
-
-   Input Parameter:
-.  eps - Eigensolver context 
-
-   Output Parameter:
-+  type       - type of orthogonalization technique
-.  refinement - type of refinement
--  eta        - parameter for selective refinement
-
-   Level: advanced
-
-.seealso: EPSOrthogonalize(), EPSSetOrthogonalization()
-@*/
-PetscErrorCode EPSGetOrthogonalization(EPS eps,EPSOrthogonalizationType *type,EPSOrthogonalizationRefinementType *refinement, PetscReal *eta)
-{
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  if (type) *type = eps->orthog_type;
-  if (refinement) *refinement = eps->orthog_ref;
-  if (eta) *eta = eps->orthog_eta;
-  PetscFunctionReturn(0);
-}
-
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSetOptionsPrefix"
 /*@C
@@ -653,13 +705,15 @@ PetscErrorCode EPSGetOrthogonalization(EPS eps,EPSOrthogonalizationType *type,EP
 
 .seealso: EPSAppendOptionsPrefix(), EPSGetOptionsPrefix()
 @*/
-PetscErrorCode EPSSetOptionsPrefix(EPS eps,char *prefix)
+PetscErrorCode EPSSetOptionsPrefix(EPS eps,const char *prefix)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
   ierr = PetscObjectSetOptionsPrefix((PetscObject)eps, prefix);CHKERRQ(ierr);
   ierr = STSetOptionsPrefix(eps->OP,prefix);CHKERRQ(ierr);
+  ierr = IPSetOptionsPrefix(eps->ip,prefix);CHKERRQ(ierr);
+  ierr = IPAppendOptionsPrefix(eps->ip,"eps_");CHKERRQ(ierr);
   PetscFunctionReturn(0);  
 }
  
@@ -683,13 +737,15 @@ PetscErrorCode EPSSetOptionsPrefix(EPS eps,char *prefix)
 
 .seealso: EPSSetOptionsPrefix(), EPSGetOptionsPrefix()
 @*/
-PetscErrorCode EPSAppendOptionsPrefix(EPS eps,char *prefix)
+PetscErrorCode EPSAppendOptionsPrefix(EPS eps,const char *prefix)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
   ierr = PetscObjectAppendOptionsPrefix((PetscObject)eps, prefix);CHKERRQ(ierr);
   ierr = STAppendOptionsPrefix(eps->OP,prefix); CHKERRQ(ierr);
+  ierr = IPSetOptionsPrefix(eps->ip,prefix);CHKERRQ(ierr);
+  ierr = IPAppendOptionsPrefix(eps->ip,"eps_");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -719,6 +775,7 @@ PetscErrorCode EPSGetOptionsPrefix(EPS eps,const char *prefix[])
   PetscErrorCode ierr;
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
+  PetscValidPointer(prefix,2);
   ierr = PetscObjectGetOptionsPrefix((PetscObject)eps, prefix);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
