@@ -1,7 +1,27 @@
 /*
       EPS routines related to the solution process.
+
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2009, Universidad Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+      
+   SLEPc is free software: you can redistribute it and/or modify it under  the
+   terms of version 3 of the GNU Lesser General Public License as published by
+   the Free Software Foundation.
+
+   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY 
+   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS 
+   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for 
+   more details.
+
+   You  should have received a copy of the GNU Lesser General  Public  License
+   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
-#include "src/eps/epsimpl.h"   /*I "slepceps.h" I*/
+
+#include "private/epsimpl.h"   /*I "slepceps.h" I*/
 
 #undef __FUNCT__  
 #define __FUNCT__ "EPSSolve"
@@ -25,35 +45,36 @@
 PetscErrorCode EPSSolve(EPS eps) 
 {
   PetscErrorCode ierr;
-  int            i;
+  PetscInt       i;
   PetscReal      re,im;
   PetscTruth     flg;
   PetscViewer    viewer;
   PetscDraw      draw;
   PetscDrawSP    drawsp;
+  STMatMode      matmode;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
 
-  ierr = PetscOptionsHasName(eps->prefix,"-eps_view_binary",&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsHasName(((PetscObject)eps)->prefix,"-eps_view_binary",&flg);CHKERRQ(ierr); 
   if (flg) {
     Mat A,B;
     ierr = STGetOperators(eps->OP,&A,&B);CHKERRQ(ierr);
-    ierr = MatView(A,PETSC_VIEWER_BINARY_(eps->comm));CHKERRQ(ierr);
-    if (B) ierr = MatView(B,PETSC_VIEWER_BINARY_(eps->comm));CHKERRQ(ierr);
+    ierr = MatView(A,PETSC_VIEWER_BINARY_(((PetscObject)eps)->comm));CHKERRQ(ierr);
+    if (B) ierr = MatView(B,PETSC_VIEWER_BINARY_(((PetscObject)eps)->comm));CHKERRQ(ierr);
   }
 
   /* reset the convergence flag from the previous solves */
   eps->reason = EPS_CONVERGED_ITERATING;
 
   if (!eps->setupcalled){ ierr = EPSSetUp(eps);CHKERRQ(ierr); }
-  ierr = STResetOperationCounters(eps->OP);
-  eps->nv = eps->ncv;
+  ierr = STResetOperationCounters(eps->OP);CHKERRQ(ierr);
+  ierr = IPResetOperationCounters(eps->ip);CHKERRQ(ierr);
   eps->evecsavailable = PETSC_FALSE;
   eps->nconv = 0;
   eps->its = 0;
   for (i=0;i<eps->ncv;i++) eps->eigr[i]=eps->eigi[i]=eps->errest[i]=0.0;
-  EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,eps->nv);
+  EPSMonitor(eps,eps->its,eps->nconv,eps->eigr,eps->eigi,eps->errest,eps->ncv);
 
   ierr = PetscLogEventBegin(EPS_Solve,eps,eps->V[0],eps->V[0],0);CHKERRQ(ierr);
 
@@ -70,6 +91,11 @@ PetscErrorCode EPSSolve(EPS eps)
       SETERRQ(1,"Wrong value of eps->solverclass");
   }
 
+  ierr = STGetMatMode(eps->OP,&matmode);CHKERRQ(ierr);
+  if (matmode == STMATMODE_INPLACE && eps->ispositive) {
+    /* Purge eigenvectors before reverting operator */
+    ierr = (*eps->ops->computevectors)(eps);CHKERRQ(ierr);    
+  }
   ierr = STPostSolve(eps->OP);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(EPS_Solve,eps,eps->V[0],eps->V[0],0);CHKERRQ(ierr);
 
@@ -87,7 +113,7 @@ PetscErrorCode EPSSolve(EPS eps)
     KSP ksp;
     Vec w;
     ierr = STGetOperators(eps->OP,PETSC_NULL,&B);CHKERRQ(ierr);
-    ierr = KSPCreate(eps->comm,&ksp);CHKERRQ(ierr);
+    ierr = KSPCreate(((PetscObject)eps)->comm,&ksp);CHKERRQ(ierr);
     ierr = KSPSetOperators(ksp,B,B,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
     ierr = MatGetVecs(B,PETSC_NULL,&w);CHKERRQ(ierr);
@@ -107,6 +133,10 @@ PetscErrorCode EPSSolve(EPS eps)
       if (eps->eigi[i] < 0) {
         eps->eigi[i] = -eps->eigi[i];
         eps->eigi[i+1] = -eps->eigi[i+1];
+        if (!eps->evecsavailable) {
+          /* the next correction only works with eigenvectors */
+          ierr = (*eps->ops->computevectors)(eps);CHKERRQ(ierr);
+        }  
         ierr = VecScale(eps->V[i+1],-1.0); CHKERRQ(ierr);
       }
       i++;
@@ -117,14 +147,14 @@ PetscErrorCode EPSSolve(EPS eps)
   /* sort eigenvalues according to eps->which parameter */
   ierr = PetscFree(eps->perm);CHKERRQ(ierr);
   if (eps->nconv > 0) {
-    ierr = PetscMalloc(sizeof(int)*eps->nconv, &eps->perm); CHKERRQ(ierr);
+    ierr = PetscMalloc(sizeof(PetscInt)*eps->nconv, &eps->perm); CHKERRQ(ierr);
     ierr = EPSSortEigenvalues(eps->nconv, eps->eigr, eps->eigi, eps->which, eps->nconv, eps->perm); CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsHasName(eps->prefix,"-eps_view",&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsHasName(((PetscObject)eps)->prefix,"-eps_view",&flg);CHKERRQ(ierr); 
   if (flg && !PetscPreLoadingOn) { ierr = EPSView(eps,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr); }
 
-  ierr = PetscOptionsHasName(eps->prefix,"-eps_plot_eigs",&flg);CHKERRQ(ierr); 
+  ierr = PetscOptionsHasName(((PetscObject)eps)->prefix,"-eps_plot_eigs",&flg);CHKERRQ(ierr); 
   if (flg) { 
     ierr = PetscViewerDrawOpen(PETSC_COMM_SELF,0,"Computed Eigenvalues",
                              PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);CHKERRQ(ierr);
@@ -165,15 +195,16 @@ PetscErrorCode EPSSolve(EPS eps)
 
    Level: intermediate
 
-   Notes:
-      During the i-th iteration this call returns i-1. If EPSSolve() is 
-      complete, then parameter "its" contains either the iteration number at
-      which convergence was successfully reached, or failure was detected.  
-      Call EPSGetConvergedReason() to determine if the solver converged or 
-      failed and why.
+   Note:
+   During the i-th iteration this call returns i-1. If EPSSolve() is 
+   complete, then parameter "its" contains either the iteration number at
+   which convergence was successfully reached, or failure was detected.  
+   Call EPSGetConvergedReason() to determine if the solver converged or 
+   failed and why.
 
+.seealso: EPSGetConvergedReason(), EPSSetTolerances()
 @*/
-PetscErrorCode EPSGetIterationNumber(EPS eps,int *its)
+PetscErrorCode EPSGetIterationNumber(EPS eps,PetscInt *its)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -210,11 +241,16 @@ PetscErrorCode EPSGetIterationNumber(EPS eps,int *its)
    Level: intermediate
 
 @*/
-PetscErrorCode EPSGetOperationCounters(EPS eps,int* ops,int* dots,int* lits)
+PetscErrorCode EPSGetOperationCounters(EPS eps,PetscInt* ops,PetscInt* dots,PetscInt* lits)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
-  STGetOperationCounters(eps->OP,ops,dots,lits);
+  ierr = STGetOperationCounters(eps->OP,ops,lits);CHKERRQ(ierr);
+  if (dots) {
+    ierr = IPGetOperationCounters(eps->ip,dots);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
@@ -238,7 +274,7 @@ PetscErrorCode EPSGetOperationCounters(EPS eps,int* ops,int* dots,int* lits)
 
 .seealso: EPSSetDimensions()
 @*/
-PetscErrorCode EPSGetConverged(EPS eps,int *nconv)
+PetscErrorCode EPSGetConverged(EPS eps,PetscInt *nconv)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -318,7 +354,7 @@ PetscErrorCode EPSGetConvergedReason(EPS eps,EPSConvergedReason *reason)
 PetscErrorCode EPSGetInvariantSubspace(EPS eps, Vec *v)
 {
   PetscErrorCode ierr;
-  int            i;
+  PetscInt       i;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -327,6 +363,9 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps, Vec *v)
   if (!eps->V) { 
     SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "EPSSolve must be called first"); 
   }
+  if (!eps->ishermitian && eps->evecsavailable) { 
+    SETERRQ(PETSC_ERR_ARG_WRONGSTATE, "EPSGetInvariantSubspace must be called before EPSGetEigenpair,EPSGetRightVector,EPSComputeRelativeError or EPSComputeResidualNorm"); 
+  }
   for (i=0;i<eps->nconv;i++) {
     ierr = VecCopy(eps->V[i],v[i]);CHKERRQ(ierr);
   }
@@ -367,7 +406,7 @@ PetscErrorCode EPSGetInvariantSubspace(EPS eps, Vec *v)
 PetscErrorCode EPSGetLeftInvariantSubspace(EPS eps, Vec *v)
 {
   PetscErrorCode ierr;
-  int            i;
+  PetscInt       i;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -405,12 +444,12 @@ PetscErrorCode EPSGetLeftInvariantSubspace(EPS eps, Vec *v)
 -  Vi   - imaginary part of eigenvector
 
    Notes:
-   If the eigenvalue is real, then eigi and Vi are set to zero. In the 
-   complex case (e.g. with BOPT=O_complex) the eigenvalue is stored 
+   If the eigenvalue is real, then eigi and Vi are set to zero. If PETSc is 
+   configured with complex scalars the eigenvalue is stored 
    directly in eigr (eigi is set to zero) and the eigenvector in Vr (Vi is 
    set to zero).
 
-   The index i should be a value between 0 and nconv (see EPSGetConverged()).
+   The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established 
    with EPSSetWhichEigenpairs().
 
@@ -419,7 +458,7 @@ PetscErrorCode EPSGetLeftInvariantSubspace(EPS eps, Vec *v)
 .seealso: EPSGetValue(), EPSGetRightVector(), EPSGetLeftVector(), EPSSolve(), 
           EPSGetConverged(), EPSSetWhichEigenpairs(), EPSGetInvariantSubspace()
 @*/
-PetscErrorCode EPSGetEigenpair(EPS eps, int i, PetscScalar *eigr, PetscScalar *eigi, Vec Vr, Vec Vi)
+PetscErrorCode EPSGetEigenpair(EPS eps, PetscInt i, PetscScalar *eigr, PetscScalar *eigi, Vec Vr, Vec Vi)
 {
   PetscErrorCode ierr;
 
@@ -453,11 +492,11 @@ PetscErrorCode EPSGetEigenpair(EPS eps, int i, PetscScalar *eigr, PetscScalar *e
 -  eigi - imaginary part of eigenvalue
 
    Notes:
-   If the eigenvalue is real, then eigi is set to zero. In the 
-   complex case (e.g. with BOPT=O_complex) the eigenvalue is stored 
+   If the eigenvalue is real, then eigi is set to zero. If PETSc is 
+   configured with complex scalars the eigenvalue is stored 
    directly in eigr (eigi is set to zero).
 
-   The index i should be a value between 0 and nconv (see EPSGetConverged()).
+   The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established 
    with EPSSetWhichEigenpairs().
 
@@ -466,9 +505,9 @@ PetscErrorCode EPSGetEigenpair(EPS eps, int i, PetscScalar *eigr, PetscScalar *e
 .seealso: EPSSolve(), EPSGetConverged(), EPSSetWhichEigenpairs(), 
           EPSGetEigenpair()
 @*/
-PetscErrorCode EPSGetValue(EPS eps, int i, PetscScalar *eigr, PetscScalar *eigi)
+PetscErrorCode EPSGetValue(EPS eps, PetscInt i, PetscScalar *eigr, PetscScalar *eigi)
 {
-  int            k;
+  PetscInt       k;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -508,11 +547,11 @@ PetscErrorCode EPSGetValue(EPS eps, int i, PetscScalar *eigr, PetscScalar *eigi)
 -  Vi   - imaginary part of eigenvector
 
    Notes:
-   If the corresponding eigenvalue is real, then Vi is set to zero. In the 
-   complex case (e.g. with BOPT=O_complex) the eigenvector is stored 
+   If the corresponding eigenvalue is real, then Vi is set to zero. If PETSc is 
+   configured with complex scalars the eigenvector is stored 
    directly in Vr (Vi is set to zero).
 
-   The index i should be a value between 0 and nconv (see EPSGetConverged()).
+   The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established 
    with EPSSetWhichEigenpairs().
 
@@ -521,10 +560,10 @@ PetscErrorCode EPSGetValue(EPS eps, int i, PetscScalar *eigr, PetscScalar *eigi)
 .seealso: EPSSolve(), EPSGetConverged(), EPSSetWhichEigenpairs(), 
           EPSGetEigenpair(), EPSGetLeftVector()
 @*/
-PetscErrorCode EPSGetRightVector(EPS eps, int i, Vec Vr, Vec Vi)
+PetscErrorCode EPSGetRightVector(EPS eps, PetscInt i, Vec Vr, Vec Vi)
 {
   PetscErrorCode ierr;
-  int            k;
+  PetscInt       k;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -541,20 +580,20 @@ PetscErrorCode EPSGetRightVector(EPS eps, int i, Vec Vr, Vec Vi)
   if (!eps->perm) k = i;
   else k = eps->perm[i];
 #ifdef PETSC_USE_COMPLEX
-  if (Vr) { ierr = VecCopy(eps->AV[k], Vr); CHKERRQ(ierr); }
+  if (Vr) { ierr = VecCopy(eps->V[k], Vr); CHKERRQ(ierr); }
   if (Vi) { ierr = VecSet(Vi,0.0); CHKERRQ(ierr); }
 #else
   if (eps->eigi[k] > 0) { /* first value of conjugate pair */
-    if (Vr) { ierr = VecCopy(eps->AV[k], Vr); CHKERRQ(ierr); }
-    if (Vi) { ierr = VecCopy(eps->AV[k+1], Vi); CHKERRQ(ierr); }
+    if (Vr) { ierr = VecCopy(eps->V[k], Vr); CHKERRQ(ierr); }
+    if (Vi) { ierr = VecCopy(eps->V[k+1], Vi); CHKERRQ(ierr); }
   } else if (eps->eigi[k] < 0) { /* second value of conjugate pair */
-    if (Vr) { ierr = VecCopy(eps->AV[k-1], Vr); CHKERRQ(ierr); }
+    if (Vr) { ierr = VecCopy(eps->V[k-1], Vr); CHKERRQ(ierr); }
     if (Vi) { 
-      ierr = VecCopy(eps->AV[k], Vi); CHKERRQ(ierr); 
+      ierr = VecCopy(eps->V[k], Vi); CHKERRQ(ierr); 
       ierr = VecScale(Vi,-1.0); CHKERRQ(ierr); 
     }
   } else { /* real eigenvalue */
-    if (Vr) { ierr = VecCopy(eps->AV[k], Vr); CHKERRQ(ierr); }
+    if (Vr) { ierr = VecCopy(eps->V[k], Vr); CHKERRQ(ierr); }
     if (Vi) { ierr = VecSet(Vi,0.0); CHKERRQ(ierr); }
   }
 #endif
@@ -579,11 +618,11 @@ PetscErrorCode EPSGetRightVector(EPS eps, int i, Vec Vr, Vec Vi)
 -  Wi   - imaginary part of eigenvector
 
    Notes:
-   If the corresponding eigenvalue is real, then Wi is set to zero. In the 
-   complex case (e.g. with BOPT=O_complex) the eigenvector is stored 
+   If the corresponding eigenvalue is real, then Wi is set to zero. If PETSc is 
+   configured with complex scalars the eigenvector is stored 
    directly in Wr (Wi is set to zero).
 
-   The index i should be a value between 0 and nconv (see EPSGetConverged()).
+   The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established 
    with EPSSetWhichEigenpairs().
 
@@ -592,10 +631,10 @@ PetscErrorCode EPSGetRightVector(EPS eps, int i, Vec Vr, Vec Vi)
 .seealso: EPSSolve(), EPSGetConverged(), EPSSetWhichEigenpairs(), 
           EPSGetEigenpair(), EPSGetLeftVector()
 @*/
-PetscErrorCode EPSGetLeftVector(EPS eps, int i, Vec Wr, Vec Wi)
+PetscErrorCode EPSGetLeftVector(EPS eps, PetscInt i, Vec Wr, Vec Wi)
 {
   PetscErrorCode ierr;
-  int            k;
+  PetscInt       k;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -616,20 +655,20 @@ PetscErrorCode EPSGetLeftVector(EPS eps, int i, Vec Wr, Vec Wi)
   if (!eps->perm) k = i;
   else k = eps->perm[i];
 #ifdef PETSC_USE_COMPLEX
-  if (Wr) { ierr = VecCopy(eps->AW[k], Wr); CHKERRQ(ierr); }
+  if (Wr) { ierr = VecCopy(eps->W[k], Wr); CHKERRQ(ierr); }
   if (Wi) { ierr = VecSet(Wi,0.0); CHKERRQ(ierr); }
 #else
   if (eps->eigi[k] > 0) { /* first value of conjugate pair */
-    if (Wr) { ierr = VecCopy(eps->AW[k], Wr); CHKERRQ(ierr); }
-    if (Wi) { ierr = VecCopy(eps->AW[k+1], Wi); CHKERRQ(ierr); }
+    if (Wr) { ierr = VecCopy(eps->W[k], Wr); CHKERRQ(ierr); }
+    if (Wi) { ierr = VecCopy(eps->W[k+1], Wi); CHKERRQ(ierr); }
   } else if (eps->eigi[k] < 0) { /* second value of conjugate pair */
-    if (Wr) { ierr = VecCopy(eps->AW[k-1], Wr); CHKERRQ(ierr); }
+    if (Wr) { ierr = VecCopy(eps->W[k-1], Wr); CHKERRQ(ierr); }
     if (Wi) { 
-      ierr = VecCopy(eps->AW[k], Wi); CHKERRQ(ierr); 
+      ierr = VecCopy(eps->W[k], Wi); CHKERRQ(ierr); 
       ierr = VecScale(Wi,-1.0); CHKERRQ(ierr); 
     }
   } else { /* real eigenvalue */
-    if (Wr) { ierr = VecCopy(eps->AW[k], Wr); CHKERRQ(ierr); }
+    if (Wr) { ierr = VecCopy(eps->W[k], Wr); CHKERRQ(ierr); }
     if (Wi) { ierr = VecSet(Wi,0.0); CHKERRQ(ierr); }
   }
 #endif
@@ -661,7 +700,7 @@ PetscErrorCode EPSGetLeftVector(EPS eps, int i, Vec Wr, Vec Wi)
 
 .seealso: EPSComputeRelativeError()
 @*/
-PetscErrorCode EPSGetErrorEstimate(EPS eps, int i, PetscReal *errest)
+PetscErrorCode EPSGetErrorEstimate(EPS eps, PetscInt i, PetscReal *errest)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -700,7 +739,7 @@ PetscErrorCode EPSGetErrorEstimate(EPS eps, int i, PetscReal *errest)
 
 .seealso: EPSComputeRelativeErrorLeft()
 @*/
-PetscErrorCode EPSGetErrorEstimateLeft(EPS eps, int i, PetscReal *errest)
+PetscErrorCode EPSGetErrorEstimateLeft(EPS eps, PetscInt i, PetscReal *errest)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(eps,EPS_COOKIE,1);
@@ -736,7 +775,7 @@ PetscErrorCode EPSGetErrorEstimateLeft(EPS eps, int i, PetscReal *errest)
    If k=0 then the residual norm is computed as ||Ax||_2.
 
    Notes:
-   The index i should be a value between 0 and nconv (see EPSGetConverged()).
+   The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established 
    with EPSSetWhichEigenpairs().
 
@@ -744,7 +783,7 @@ PetscErrorCode EPSGetErrorEstimateLeft(EPS eps, int i, PetscReal *errest)
 
 .seealso: EPSSolve(), EPSGetConverged(), EPSSetWhichEigenpairs()
 @*/
-PetscErrorCode EPSComputeResidualNorm(EPS eps, int i, PetscReal *norm)
+PetscErrorCode EPSComputeResidualNorm(EPS eps, PetscInt i, PetscReal *norm)
 {
   PetscErrorCode ierr;
   Vec            u, v, w, xr, xi;
@@ -819,7 +858,7 @@ PetscErrorCode EPSComputeResidualNorm(EPS eps, int i, PetscReal *norm)
    If k=0 then the residual norm is computed as ||y'A||_2.
 
    Notes:
-   The index i should be a value between 0 and nconv (see EPSGetConverged()).
+   The index i should be a value between 0 and nconv-1 (see EPSGetConverged()).
    Eigenpairs are indexed according to the ordering criterion established 
    with EPSSetWhichEigenpairs().
 
@@ -827,7 +866,7 @@ PetscErrorCode EPSComputeResidualNorm(EPS eps, int i, PetscReal *norm)
 
 .seealso: EPSSolve(), EPSGetConverged(), EPSSetWhichEigenpairs()
 @*/
-PetscErrorCode EPSComputeResidualNormLeft(EPS eps, int i, PetscReal *norm)
+PetscErrorCode EPSComputeResidualNormLeft(EPS eps, PetscInt i, PetscReal *norm)
 {
   PetscErrorCode ierr;
   Vec            u, v, w, xr, xi;
@@ -906,7 +945,7 @@ PetscErrorCode EPSComputeResidualNormLeft(EPS eps, int i, PetscReal *norm)
 
 .seealso: EPSSolve(), EPSComputeResidualNorm(), EPSGetErrorEstimate()
 @*/
-PetscErrorCode EPSComputeRelativeError(EPS eps, int i, PetscReal *error)
+PetscErrorCode EPSComputeRelativeError(EPS eps, PetscInt i, PetscReal *error)
 {
   PetscErrorCode ierr;
   Vec            xr, xi;  
@@ -979,7 +1018,7 @@ PetscErrorCode EPSComputeRelativeError(EPS eps, int i, PetscReal *error)
 
 .seealso: EPSSolve(), EPSComputeResidualNormLeft(), EPSGetErrorEstimateLeft()
 @*/
-PetscErrorCode EPSComputeRelativeErrorLeft(EPS eps, int i, PetscReal *error)
+PetscErrorCode EPSComputeRelativeErrorLeft(EPS eps, PetscInt i, PetscReal *error)
 {
   PetscErrorCode ierr;
   Vec            xr, xi;  
@@ -1053,12 +1092,12 @@ PetscErrorCode EPSComputeRelativeErrorLeft(EPS eps, int i, PetscReal *error)
 
    Level: developer
 
-.seealso: EPSDenseNHEPSorted(), EPSSetWhichEigenpairs()
+.seealso: EPSSortEigenvaluesReal(), EPSSetWhichEigenpairs()
 @*/
-PetscErrorCode EPSSortEigenvalues(int n,PetscScalar *eig,PetscScalar *eigi,EPSWhich which,int nev,int *permout)
+PetscErrorCode EPSSortEigenvalues(PetscInt n,PetscScalar *eig,PetscScalar *eigi,EPSWhich which,PetscInt nev,PetscInt *permout)
 {
   PetscErrorCode ierr;
-  int            i;
+  PetscInt       i;
   PetscInt       *perm;
   PetscReal      *values;
 
@@ -1109,10 +1148,10 @@ PetscErrorCode EPSSortEigenvalues(int n,PetscScalar *eig,PetscScalar *eigi,EPSWh
       if (eig[permout[i]] == eig[permout[i+1]] &&
           eigi[permout[i]] == -eigi[permout[i+1]] &&
           eigi[permout[i]] < 0.0) {
-        int tmp;
+        PetscInt tmp;
         SWAP(permout[i], permout[i+1], tmp);
       }
-    i++;
+      i++;
     }
   }
 #endif
@@ -1122,6 +1161,73 @@ PetscErrorCode EPSSortEigenvalues(int n,PetscScalar *eig,PetscScalar *eigi,EPSWh
   PetscFunctionReturn(0);
 }
 
+#undef __FUNCT__  
+#define __FUNCT__ "EPSSortEigenvaluesReal"
+/*@
+   EPSSortEigenvaluesReal - Sorts a list of eigenvalues according to a certain
+   criterion (version for real eigenvalues only).
+
+   Not Collective
+
+   Input Parameters:
++  n     - number of eigenvalue in the list
+.  eig   - pointer to the array containing the eigenvalues (real)
+.  which - sorting criterion
+-  nev   - number of wanted eigenvalues
+
+   Output Parameter:
+.  permout - resulting permutation
+
+   Workspace:
+.  work - workspace for storing n real values and n integer values
+
+   Notes:
+   The result is a list of indices in the original eigenvalue array 
+   corresponding to the first nev eigenvalues sorted in the specified
+   criterion
+
+   Level: developer
+
+.seealso: EPSSortEigenvalues(), EPSSetWhichEigenpairs()
+@*/
+PetscErrorCode EPSSortEigenvaluesReal(PetscInt n,PetscReal *eig,EPSWhich which,PetscInt nev,PetscInt *permout,PetscReal *work)
+{
+  PetscErrorCode ierr;
+  PetscInt            i;
+  PetscReal      *values = work;
+  PetscInt       *perm = (PetscInt*)(work+n);
+
+  PetscFunctionBegin;
+  for (i=0; i<n; i++) { perm[i] = i;}
+
+  switch(which) {
+    case EPS_LARGEST_MAGNITUDE:
+    case EPS_SMALLEST_MAGNITUDE:
+      for (i=0; i<n; i++) { values[i] = PetscAbsReal(eig[i]); }
+      break;
+    case EPS_LARGEST_REAL:
+    case EPS_SMALLEST_REAL:
+      for (i=0; i<n; i++) { values[i] = eig[i]; }
+      break;
+    default: SETERRQ(1,"Wrong value of which");
+  }
+
+  ierr = PetscSortRealWithPermutation(n,values,perm);CHKERRQ(ierr);
+
+  switch(which) {
+    case EPS_LARGEST_MAGNITUDE:
+    case EPS_LARGEST_REAL:
+      for (i=0; i<nev; i++) { permout[i] = perm[n-1-i]; }
+      break;
+    case EPS_SMALLEST_MAGNITUDE:
+    case EPS_SMALLEST_REAL:
+      for (i=0; i<nev; i++) { permout[i] = perm[i]; }
+      break;
+    default: SETERRQ(1,"Wrong value of which");
+  }
+  PetscFunctionReturn(0);
+}
+
 #undef __FUNCT__  
 #define __FUNCT__ "EPSGetStartVector"
 /*@
@@ -1141,8 +1247,9 @@ PetscErrorCode EPSSortEigenvalues(int n,PetscScalar *eig,PetscScalar *eigi,EPSWh
    Notes:
    The start vector is computed from another vector: for the first step (i=0),
    the initial vector is used (see EPSGetInitialVector()); otherwise a random
-   vector is created. Then this vector is forced to be in the range of OP and
-   orthonormalized with respect to all V-vectors up to i-1.
+   vector is created. Then this vector is forced to be in the range of OP (only
+   for generalized definite problems) and orthonormalized with respect to all
+   V-vectors up to i-1.
 
    The flag breakdown is set to true if either i=0 and the vector belongs to the
    deflation space, or i>0 and the vector is linearly dependent with respect
@@ -1156,7 +1263,7 @@ PetscErrorCode EPSSortEigenvalues(int n,PetscScalar *eig,PetscScalar *eigi,EPSWh
 .seealso: EPSGetInitialVector()
 
 @*/
-PetscErrorCode EPSGetStartVector(EPS eps,int i,Vec vec,PetscTruth *breakdown)
+PetscErrorCode EPSGetStartVector(EPS eps,PetscInt i,Vec vec,PetscTruth *breakdown)
 {
   PetscErrorCode ierr;
   PetscReal      norm;
@@ -1175,18 +1282,22 @@ PetscErrorCode EPSGetStartVector(EPS eps,int i,Vec vec,PetscTruth *breakdown)
     ierr = SlepcVecSetRandom(w);CHKERRQ(ierr);
   }
 
-  /* Force the vector to be in the range of OP */
-  ierr = STApply(eps->OP,w,vec);CHKERRQ(ierr);
+  /* Force the vector to be in the range of OP for definite generalized problems */
+  if (eps->ispositive) {
+    ierr = STApply(eps->OP,w,vec);CHKERRQ(ierr);
+  } else {
+    ierr = VecCopy(w,vec);CHKERRQ(ierr);
+  }
 
   /* Orthonormalize the vector with respect to previous vectors */
-  ierr = EPSOrthogonalize(eps,i+eps->nds,PETSC_NULL,eps->DSV,vec,PETSC_NULL,&norm,&lindep);CHKERRQ(ierr);
+  ierr = IPOrthogonalize(eps->ip,i+eps->nds,PETSC_NULL,eps->DSV,vec,PETSC_NULL,&norm,&lindep,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
   if (breakdown) *breakdown = lindep;
   else if (lindep || norm == 0.0) {
     if (i==0) { SETERRQ(1,"Initial vector is zero or belongs to the deflation space"); } 
     else { SETERRQ(1,"Unable to generate more start vectors"); }
   }
   
-  ierr = VecScale(vec,1/norm);CHKERRQ(ierr);
+  ierr = VecScale(vec,1.0/norm);CHKERRQ(ierr);
 
   if (i!=0) {
     ierr = VecDestroy(w);CHKERRQ(ierr);
@@ -1223,7 +1334,7 @@ PetscErrorCode EPSGetStartVector(EPS eps,int i,Vec vec,PetscTruth *breakdown)
 .seealso: EPSGetLeftInitialVector()
 
 @*/
-PetscErrorCode EPSGetLeftStartVector(EPS eps,int i,Vec vec)
+PetscErrorCode EPSGetLeftStartVector(EPS eps,PetscInt i,Vec vec)
 {
   PetscErrorCode ierr;
   PetscTruth     breakdown;
@@ -1247,7 +1358,7 @@ PetscErrorCode EPSGetLeftStartVector(EPS eps,int i,Vec vec)
   ierr = STApplyTranspose(eps->OP,w,vec);CHKERRQ(ierr);
 
   /* Orthonormalize the vector with respect to previous vectors */
-  ierr = EPSOrthogonalize(eps,i,PETSC_NULL,eps->W,vec,PETSC_NULL,&norm,&breakdown);CHKERRQ(ierr);
+  ierr = IPOrthogonalize(eps->ip,i,PETSC_NULL,eps->W,vec,PETSC_NULL,&norm,&breakdown,PETSC_NULL,PETSC_NULL);CHKERRQ(ierr);
   if (breakdown) {
     if (i==0) { SETERRQ(1,"Left initial vector is zero"); }
     else { SETERRQ(1,"Unable to generate more left start vectors"); }
@@ -1260,4 +1371,3 @@ PetscErrorCode EPSGetLeftStartVector(EPS eps,int i,Vec vec)
 
   PetscFunctionReturn(0);
 }
-
